package org.deltadebugging.ddstate.launcher;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.deltadebugging.ddstate.launcher.client.IDDLauncher;
import org.deltadebugging.ddstate.launcher.client.SimpleDDLauncher;
import org.deltadebugging.ddstate.launcher.client.cts.CTSRunListenerManager;
import org.deltadebugging.ddstate.launcher.client.cts.ICTSRunListenerManager;
import org.deltadebugging.ddstate.launcher.ddrun.DDRunListenerManager;
import org.deltadebugging.ddstate.launcher.ddrun.IDDRunListenerManager;
import org.deltadebugging.ddstate.logger.DDLogger;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class DDStateLaunchPlugin extends AbstractUIPlugin {
	// The shared instance.
	private static DDStateLaunchPlugin plugin;

	// Resource bundle.
	private ResourceBundle resourceBundle;

	// use for tracing
	public final static boolean DEBUG;

	public final static boolean DEBUG_REMOTE_TEST_RUNNER;

	public final static boolean DEBUG_STACKTRACE_MATCHER;

	public final static boolean DEBUG_CTS_LAUNCHER;

	public final static boolean DEBUG_DELTA_TESTER;
    
    public final static boolean DEBUG_DELTA_PROCESSOR;

	public final static String PLUGIN_ID = "org.deltadebugging.ddstate.launcher";

	public static final String ID_EXTENSION_POINT_DDRUN_LISTENERS = PLUGIN_ID
			+ ".ddRunListeners"; //$NON-NLS-1$

	public static final String ID_EXTENSION_POINT_CTSRUN_LISTENERS = PLUGIN_ID
			+ ".ctsRunListeners"; //$NON-NLS-1$

	static {
		DEBUG = "true".equalsIgnoreCase(Platform.getDebugOption(PLUGIN_ID
				+ "/debug"));
		DEBUG_REMOTE_TEST_RUNNER = "true".equalsIgnoreCase(Platform
				.getDebugOption(PLUGIN_ID + "/debug/RemoteTestRunner"));
		DEBUG_STACKTRACE_MATCHER = "true".equalsIgnoreCase(Platform
				.getDebugOption(PLUGIN_ID + "/debug/StackTraceMatcher"));
		DEBUG_CTS_LAUNCHER = "true".equalsIgnoreCase(Platform
				.getDebugOption(PLUGIN_ID + "/debug/CTSLauncher"));
		DEBUG_DELTA_TESTER = "true".equalsIgnoreCase(Platform
				.getDebugOption(PLUGIN_ID + "/debug/DeltaTester"));
        DEBUG_DELTA_PROCESSOR = "true".equalsIgnoreCase(Platform
                .getDebugOption(PLUGIN_ID + "/debug/DeltaProcessor"));
	}

	/**
	 * The constructor.
	 */
	public DDStateLaunchPlugin() {
		super();
		plugin = this;
		try {
			this.resourceBundle = ResourceBundle
					.getBundle("org.deltadebugging.ddstate.junit.DDStateJunitPluginResources");
		} catch (MissingResourceException x) {
			this.resourceBundle = null;
		}
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
	}

	/**
	 * Returns the shared instance.
	 */
	public static DDStateLaunchPlugin getDefault() {
		return plugin;
	}
    
    /**
     * Returns the standard display to be used. The method first checks, if the
     * thread calling this method has an associated display. If so, this display
     * is returned. Otherwise the method returns the default display.
     */
    public static Display getStandardDisplay() {
        Display display = Display.getCurrent();
        if (display == null) {
            display = Display.getDefault();
        }
        return display;
    }

	/**
	 * Returns the string from the plugin's resource bundle, or 'key' if not
	 * found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = DDStateLaunchPlugin.getDefault()
				.getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return this.resourceBundle;
	}

	/**
	 * Return a logger for the given tracer class.
	 * 
	 * @param tracer
	 *            the tracer is the class that have to log or trace messages.
	 * @return a logger for the given tracer class.
	 */
	public static DDLogger getLogger(Class tracer) {
		return new DDLogger(DDStateLaunchPlugin.getDefault(), tracer);
	}

	/**
	 * Returns the instance of a IDDRunListenerManager. The manager has the
	 * function to manage all registered IDDRunListener and inform them on
	 * changes of a delta debugging run.
	 * 
	 * @return an instance of IDDRunListenerManager
	 * 
	 * @see IDDRunListenerManager
	 */
	public static IDDRunListenerManager getDDRunListenerManager() {
		return DDRunListenerManager.getInstance();
	}

	/**
	 * Returns the instance of a ICTSRunListenerManager. The manager has the
	 * function to manage all registered ICTSRunListener and inform them on
	 * changes of a cause transition run.
	 * 
	 * @return an instance of ICTSRunListenerManager
	 * 
	 * @see ICTSRunListenerManager
	 */
	public static ICTSRunListenerManager getCTSRunListenerManager() {
		return CTSRunListenerManager.getInstance();
	}

	/**
	 * Returns a new instance of the dd launcher. Use the launcher to start the
	 * delta debugging on java states.
	 * 
	 * @return a new instance of the dd launcher.
	 */
	public static IDDLauncher getDDLauncher() {
		return new SimpleDDLauncher();
	}

	/**
	 * Show the given status in an error dialog with the given title.
	 * 
	 * @param status
	 *            the status you want to show in the error dialog.
	 * @param title
	 *            the title of the error dialog
	 */
	public static void showErrorDialog(final IStatus status, final String title) {
		getDisplay().asyncExec(new Runnable() {

			public void run() {
				ErrorDialog.openError(getWorkbenchWindow().getShell(), title,
						null, status);
			}
		});

	}

	/**
	 * Returns the currently active window. Returns null if there is no active
	 * workbench window. Returns null if called from a non-UI thread.
	 * 
	 * @return the active workbench window, or null if there is no active
	 *         workbench window or if called from a non-UI thread
	 */
	public static IWorkbenchWindow getWorkbenchWindow() {
		IWorkbenchWindow result = null;

		IWorkbench wb = PlatformUI.getWorkbench();
		result = wb.getActiveWorkbenchWindow();

		return result;
	}

	/**
	 * Returns the current display if it availabe, otherwise the default
	 * display.
	 * 
	 * @return the current display if it availabe, otherwise the default
	 *         display.
	 */
	public static Display getDisplay() {
		Display result = null;

		result = Display.getCurrent();

		// may be null if outside of the ui thread
		if (result == null) {
			result = Display.getDefault();
		}

		return result;
	}

	private static IExtBreakpointManager fBreakpointManager = null;

	/**
	 * Returns the breakpoint manager used by this plugin to manage the
	 * breapoints and the breakpoint listeners.
	 * 
	 * @return the breakpoint manager to manage the breakpoints and breakpoint
	 *         managers.
	 */
	public static IExtBreakpointManager getBreakpointManager() {
		if (fBreakpointManager == null) {
			fBreakpointManager = new BreakpointManager();
		}

		return fBreakpointManager;
	}
}