/*
 * $$Id: IExtBreakpointManager.java,v 1.4 2005/08/21 13:14:05 lehmann Exp $$
 * 
 * Copyright (C) ${year} Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on ${date} ${time} by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher;

import org.deltadebugging.ddstate.launcher.matcher.breakpoint.IBreakpointLocation;
import org.deltadebugging.ddstate.memorygraph.IMethodIdentifier;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.IBreakpointManager;
import org.eclipse.jdt.debug.core.IJavaBreakpoint;
import org.eclipse.jdt.debug.core.IJavaLineBreakpoint;
import org.eclipse.jdt.debug.core.IJavaMethodEntryBreakpoint;

/**
 * The extended breakpoint manager extends the plattfrom defined breakpoint
 * manager with some usefull functionality for the delta debugging launcher.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IExtBreakpointManager extends IBreakpointManager {

	/**
	 * Register the given breakpoint at this breakpoint manager if it is not
	 * allways registered.
	 * 
	 * @param breakpoint
	 *            the breakpoint to register at this manager.
	 * @throws CoreException
	 *             if an exception occure while register the given breakpoint.
	 */
	public void registerBreakpoint(IJavaBreakpoint breakpoint)
			throws CoreException;

	/**
	 * Unregister the given breakpoint at this breakpoint manager if it is
	 * actually registered.
	 * 
	 * @param breakpoint
	 *            the breakpoint to unregister at this manager.
	 * @param delete
	 *            if the breakpoint should always delete after it was
	 *            unregisterd.
	 * @throws CoreException
	 *             if an exception occure while unregister or delete the given
	 *             breakpoint.
	 */
	public void unregisterBreakpoint(IJavaBreakpoint breakpoint, boolean delete)
			throws CoreException;

	/**
	 * Delete the given breakpoint if the breakpoint is not <code>null</code>.
	 * 
	 * @param breakpoint
	 *            breakpoint to delete
	 * @throws CoreException
	 *             if an exception occure while deleting the given breakpoint.
	 */
	public void deleteExistingBreakpoint(IJavaBreakpoint breakpoint)
			throws CoreException;

	/**
	 * Delete all given breakpoints.
	 * 
	 * @param breakpoints
	 *            breakpoint to delete
	 * @throws CoreException
	 *             if an exception occure while deleting one of the given
	 *             breakpoints.
	 */
	public void deleteExistingBreakpoints(IJavaBreakpoint[] breakpoints)
			throws CoreException;

	/**
	 * Disable all given existing java breakpoints and return an array of all
	 * disabled java breakpoints.
	 * 
	 * @return array of all disabled java breakpoints.
	 * @throws CoreException
	 *             while an excepiton occure on disable all existing
	 *             breakpoints.
	 */
	public IJavaBreakpoint[] disableAllExistingBreakpoints()
			throws CoreException;

	/**
	 * Enable all breakpoints of the given array of breakpoints.
	 * 
	 * @param breakpoints
	 *            array of existing java breakpoints.
	 * @throws CoreException
	 *             if unable to set the associated attribute on this
	 *             breakpoint's underlying marker.
	 */
	public void enabledBreakpoints(IJavaBreakpoint[] breakpoints)
			throws CoreException;

	/**
	 * Set a line breakpoint in the given type name at the given line.
	 * 
	 * @param typeName
	 *            the fully qualified name of the type the breakpoint is to be
	 *            installed in. If the breakpoint is to be installed in an inner
	 *            type, it is sufficient to provide the name of the top level
	 *            enclosing type. If an inner class name is specified, it should
	 *            be formatted as the associated class file name (i.e. with
	 *            <code>$</code>). For example,
	 *            <code>example.SomeClass$InnerType</code>, could be
	 *            specified, but <code>example.SomeClass</code> is sufficient.
	 * @param line
	 *            the lineNumber on which the breakpoint is set - line numbers
	 *            are 1 based, associated with the source file in which the
	 *            breakpoint is set.
	 * @param hitCount
	 *            the number of times the breakpoint will be hit before
	 *            suspending execution - 0 if it should always suspend.
	 * @param register
	 *            whether to add this breakpoint to the breakpoint manager
	 * @return the line breakpoint at the specified program point.
	 * @throws CoreException
	 *             if something fails on creating or enable the line breakpoint.
	 */
	public abstract IJavaLineBreakpoint createLineBreakPoint(String typeName,
			int line, int hitCount, boolean register) throws CoreException;

	/**
	 * Creates and returns a method entry breakpoint with the specified
	 * criteria. A method entry breakpoint will only be installed for methods
	 * that have executable code (i.e. will not work for native methods).
	 * 
	 * @param methodIdentifier
	 *            the identifier of the method this breakpoints suspends
	 *            execution in
	 * @param hitCount
	 *            the number of times the breakpoint will be hit before
	 *            suspending execution - 0 if it should always suspend
	 * @param register
	 *            whether to add this breakpoint to the breakpoint manager
	 * @return a method entry breakpoint
	 * @exception CoreException
	 *                If this method fails. Reasons include:
	 *                <ul>
	 *                <li>Failure creating underlying marker. The exception's
	 *                status contains the underlying exception responsible for
	 *                the failure.</li>
	 *                </ul>
	 */
	public abstract IJavaMethodEntryBreakpoint createMethodEntryBreakpoint(
			IMethodIdentifier methodIdentifier, int hitCount, boolean register)
			throws CoreException;

	/**
	 * Create and return a breakpoint for the given breakpoint description.
	 * 
	 * @param breakpointLocation
	 *            the location for the breakpoint you want to create.
	 * @param register
	 *            whether to add this breakpoint to the breakpoint manager
	 * @return a java breakpoint
	 * @throws CoreException
	 *             If this method fails. Reasons include:
	 *             <ul>
	 *             <li>Failure creating underlying marker. The exception's
	 *             status contains the underlying exception responsible for the
	 *             failure.</li>
	 *             </ul>
	 */
	public abstract IJavaBreakpoint createBreakpoint(
			IBreakpointLocation breakpointLocation, boolean register)
			throws CoreException;
}