/*
 * $Id: RuntimeJarUtil.java,v 1.5 2006/01/10 19:01:18 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Aug 4, 2005 10:43:31 AM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.client;

import org.deltadebugging.ddstate.launcher.DDStateLaunchPlugin;
import org.deltadebugging.ddstate.launcher.bcel.instrumenter.IClassInstrumenter;
import org.deltadebugging.ddstate.launcher.bcel.instrumenter.JarInstrumenter;
import org.deltadebugging.ddstate.launcher.bcel.instrumenter.ModifiedJarCreationException;
import org.deltadebugging.ddstate.launcher.server.instrumenter.CreateInstanceInstrumenter;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jdt.launching.LibraryLocation;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

/**
 * This util check the existens for the modified runtime jar. It also provides
 * functionality to create a modified runtime jar for a given vm install. The
 * modified runtime jar will used to create empty objects of any class in the
 * runtime jar. Therefor the rt.jar will be adpated using byte code engeneering.
 * A new constructor with a unique signature will be added to every class in the
 * runtime jar. Also the static block will surounded by an if construct. This
 * support us to create an empty object, because we can disable the excecution
 * of the code in the static block. <br>
 * E.g. to create an empty object of the type hashmap you have to execute the
 * follwing code: <br>
 * <code>
 * org.deltadebugging.ddstate.launcher.server.instrumenter.rtclass.ClassController.WITH_STATIC_BLOCK = false; <br>
 * HashMap myMap = new HashMap(org.deltadebugging.ddstate.launcher.server.instrumenter.rtclass.ClassController.getDefault()); <br>
 * org.deltadebugging.ddstate.launcher.server.instrumenter.rtclass.ClassController.WITH_STATIC_BLOCK = true;
 * </code>
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class RuntimeJarUtil {

    private final static String MODIFIED_RT_JAR_SUBDIR = "modsdk";
    private final static String MODIFIED_RT_JAR_NAME = "mod_rt.jar";

    private RuntimeJarUtil() {
        // Only call the staic methods
    }

    /**
     * Check if the modified runtime jar for the given vm install exists. If it
     * doesen't exists the user will be asked to create the modified runtime jar
     * within a dialog.
     * 
     * @param vmInstall
     *            the particular installation of a VM.
     * @param monitor
     * @return true if either the modified runtime jar already exists or it
     *         would successful created, otherwise false.
     * @throws ModifiedJarCreationException
     *             if something fails on the creation of the modified runtime
     *             jar. This cause e.g. includes that is not enough memory to
     *             create the jar.
     */
    public static boolean checkAndCreate(IVMInstall vmInstall,
            IProgressMonitor monitor) throws ModifiedJarCreationException {
        boolean result = true;

        try {
            monitor.beginTask("Create modified runtime jar", 100);

            if (!RuntimeJarUtil.existModifiedRuntimeJar(vmInstall)) {

                ModRTMessDialogRunnable runnable = new ModRTMessDialogRunnable(
                        vmInstall);
                Display.getDefault().syncExec(runnable);

                if (runnable.getResult() == ModRTMessDialogRunnable.CONTINUE) {
                    RuntimeJarUtil.createModifiedRuntimeJar(vmInstall,
                            new SubProgressMonitor(monitor, 100));

                } else {
                    result = false;
                }

            } else {
                monitor.worked(100);

            }

        } finally {
            monitor.done();
        }

        return result;
    }

    /**
     * Create a modified runtime jar for the given vm install if it not exists.
     * 
     * @param vmInstall
     *            the particular installation of a VM.
     * @param monitor
     * @throws ModifiedJarCreationException
     *             if something fails on the creation of the modified runtime
     *             jar. This cause e.g. includes that is not enough memory to
     *             create the jar.
     */
    public static void createModifiedRuntimeJar(IVMInstall vmInstall,
            IProgressMonitor monitor) throws ModifiedJarCreationException {

        if (!existModifiedRuntimeJar(vmInstall)) {
            IPath modRTJarPath = getModRTJarLocation(vmInstall);

            LibraryLocation[] locations = JavaRuntime
                    .getLibraryLocations(vmInstall);

            IPath rtJarPath = null;
            for (int i = 0; i < locations.length; i++) {
                LibraryLocation location = locations[i];

                System.out.println(location.getSystemLibraryPath().toFile()
                        .getName());
                String jarName = location.getSystemLibraryPath().toFile()
                        .getName();
                if (jarName.equals("rt.jar") || jarName.equals("classes.jar")) {
                    rtJarPath = location.getSystemLibraryPath();
                    break;
                }
            }

            IClassInstrumenter classInstrumenter = new CreateInstanceInstrumenter();
            JarInstrumenter.createModifiedJar(rtJarPath.toOSString(),
                    modRTJarPath.toOSString(), MODIFIED_RT_JAR_NAME,
                    classInstrumenter, monitor);

        }
    }

    /**
     * Check if the modified runtime jar exists for the given vm install.
     * 
     * @param vmInstall
     *            the particular installation of a VM.
     * @return true if the modified runtime jar exists for the given vm install,
     *         otherwise false.
     */
    public static boolean existModifiedRuntimeJar(IVMInstall vmInstall) {
        boolean result = false;

        IPath modRTJarPath = getFullModRTJarLocation(vmInstall);
        result = modRTJarPath.toFile().exists();

        return result;
    }

    /**
     * Create the path to the modified rt jar of the given vm install. This is
     * only the directory path with the filename.
     * 
     * @param vmInstall
     *            the particular installation of a VM.
     * @return the path to the modified rt jar of the given vm install.
     */
    public static IPath getModRTJarLocation(IVMInstall vmInstall) {
        IPath result = null;

        result = DDStateLaunchPlugin.getDefault().getStateLocation();
        // the subdirectory for the modified rt jars
        result = result.append(MODIFIED_RT_JAR_SUBDIR);
        // the subdirectory for the different vm's
        result = result.append("/" + vmInstall.getName());

        return result;
    }

    /**
     * Create the path to the modified rt jar of the given vm install inclusive
     * the filename.
     * 
     * @param vmInstall
     *            the particular installation of a VM.
     * @return the full qualified path to the modified rt jar of the given vm
     *         install inclusive the fillname
     */
    public static IPath getFullModRTJarLocation(IVMInstall vmInstall) {
        IPath result = null;

        result = getModRTJarLocation(vmInstall);
        result = result.append("/" + MODIFIED_RT_JAR_NAME);

        return result;
    }

    static class ModRTMessDialogRunnable implements Runnable {

        public final static int CONTINUE = 0;
        public final static int CANCEL = 1;

        private final IVMInstall fVMInstall;
        private int fResult;

        public ModRTMessDialogRunnable(IVMInstall vmInstall) {
            this.fVMInstall = vmInstall;
            this.fResult = CANCEL;
        }

        /**
         * @see java.lang.Runnable#run()
         */
        public void run() {
            MessageDialog dialog = new MessageDialog(
                    null,
                    "Modify Java Runtime Classes",
                    null,
                    "You are using DDstate for the first time with '"
                            + this.fVMInstall.getName()
                            + "'. DDstate will now instrument the runtime classes.",
                    MessageDialog.QUESTION,
                    new String[] { "Continue", "Cancel" }, CONTINUE);
            this.fResult = dialog.open();
        }

        /**
         * Returns the result of the dialog. This can be CONTINUE or CANCEL.
         * 
         * @return the result of the dialog. This can be CONTINUE or CANCEL.
         */
        public int getResult() {
            return this.fResult;
        }
    }

}
