/*
 * $Id: SearchStrategy.java,v 1.3 2005/10/25 11:10:56 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 10.09.2005 13:46:56 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.client.cts;

import org.deltadebugging.ddstate.comparator.exceptions.GraphComparatorException;
import org.deltadebugging.ddstate.launcher.client.BaseLauncher;
import org.deltadebugging.ddstate.launcher.client.TestRunnerException;
import org.deltadebugging.ddstate.launcher.client.exception.DeltaComputationException;
import org.deltadebugging.ddstate.launcher.ddrun.DDRunDescription;
import org.deltadebugging.ddstate.launcher.ddrun.IDDRunDescription;
import org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult;
import org.deltadebugging.ddstate.launcher.ddrun.IDDRunStatusCodes;
import org.deltadebugging.ddstate.launcher.matcher.IMatchPoint;
import org.deltadebugging.ddstate.launcher.tester.exception.TesterException;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * COMMENTME Comment type SearchStrategy
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann</a>
 */
public class SearchStrategy {

    /** constants for the search outcome */
    public final static int NO_OUTCOME = 1;

    public final static int RELEVANT_DELTAS_FOUND = 2;

    public final static int NO_RELEVANT_DELTAS_FOUND = 4;

    public final static int DELTA_COMPUTATION_WITH_ERRORS = 8;

    private final BaseLauncher fLauncher;

    private final DDResultCache fCache = new DDResultCache();

    /**
     * Set maximal search range are the number of match points used for one
     * search.
     */
    private int fMaxSearchRange = 3;

    public SearchStrategy(BaseLauncher launcher) {
        this.fLauncher = launcher;
    }

    public SearchStrategy(BaseLauncher launcher, int maxSearchRange) {
        this.fLauncher = launcher;
        this.fMaxSearchRange = maxSearchRange;
    }

    public void setMaxSearchRange(int maxSearchRange) {
        this.fMaxSearchRange = maxSearchRange;
    }

    public SearchResult searchSuccessfulDDRun(MatchPointInterval mpInterval,
            boolean searchForward, int acceptedOutcomes,
            IProgressMonitor monitor) throws GraphComparatorException,
            DeltaComputationException, TestRunnerException, TesterException {
        SearchResult result = new SearchResult();

        try {
            monitor
                    .beginTask("DDstate - search in space",
                            this.fMaxSearchRange);

            int searchIndex = searchForward ? 0 : mpInterval.length() - 1;
            int counter = 0;

            IMatchPoint searchPoint = null;
            IDDRunResult ddRunResult = null;

            while (!monitor.isCanceled() && counter < this.fMaxSearchRange
                    && searchIndex >= 0 && searchIndex < mpInterval.length()
                    && result.searchOutcome == NO_OUTCOME) {

                result.resultIndex = searchIndex;

                searchPoint = mpInterval.getMatchPointAtIndex(searchIndex);
                ddRunResult = this.computeRelevantDeltas(searchPoint, monitor);

                int ddRunOutcome = this.evaluateDDRunResult(ddRunResult);
                if ((ddRunOutcome & acceptedOutcomes) > 0) {
                    result.ddRunResult = ddRunResult;
                    result.searchOutcome = ddRunOutcome;
                }

                searchIndex += searchForward ? 1 : -1;

                counter++;
                monitor.worked(1);
            }
            
        } finally {
            monitor.done();
        }

        return result;
    }

    private int evaluateDDRunResult(IDDRunResult ddRunResult) {
        int result = NO_OUTCOME;

        if (ddRunResult != null) {
            if (ddRunResult.hasErrorStatus()) {
                result = DELTA_COMPUTATION_WITH_ERRORS;
            } else if (ddRunResult.hasStatus(IDDRunStatusCodes.NO_DELTAS_FOUND)) {
                result = NO_RELEVANT_DELTAS_FOUND;
            } else if (ddRunResult.getMinimizedDeltaSet().size() == 0) {
                result = NO_RELEVANT_DELTAS_FOUND;
            } else if (ddRunResult.getMinimizedDeltaSet().size() > 0) {
                result = RELEVANT_DELTAS_FOUND;
            }
        }

        return result;
    }

    /**
     * Compute the dd run result for the given matchpoint. If the result was
     * already computed the cached result is returned.
     * 
     * @param matchPoint
     * @param monitor
     * @return the dd run result for the given matchpoint.
     * @throws GraphComparatorException
     * @throws DeltaComputationException
     * @throws TestRunnerException
     * @throws TesterException
     */
    private IDDRunResult computeRelevantDeltas(IMatchPoint matchPoint,
            IProgressMonitor monitor) throws GraphComparatorException,
            DeltaComputationException, TestRunnerException, TesterException {
        IDDRunResult result = null;

        if (this.fCache.existDDResultForKey(matchPoint)) {
            return this.fCache.getDDResult(matchPoint);
        }

        // create and register the breakpoints from the match point
        IDDRunDescription ddRunDesc = new DDRunDescription(this.fLauncher
                .getLaunchDescription(), matchPoint);

        result = this.fLauncher.computeRelevantDeltas(ddRunDesc, monitor);

        this.fCache.addDDResult(matchPoint, result);

        return result;
    }

    public class SearchResult {

        public int searchOutcome = SearchStrategy.NO_OUTCOME;

        public MatchPointInterval mpInterval = null;

        public int resultIndex = -1;

        public IDDRunResult ddRunResult = null;

    }

}
