/*
 * $Id: TestRunnerJob.java,v 1.2 2005/09/13 11:07:27 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 07.06.2005 10:54:31 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.client.testrunner;

import org.deltadebugging.ddstate.launcher.DDStateLaunchPlugin;
import org.deltadebugging.ddstate.launcher.client.IExtTestRunner;
import org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription;
import org.deltadebugging.ddstate.launcher.client.TestRunnerException;
import org.deltadebugging.ddstate.launcher.tester.predicate.IPredicate;
import org.deltadebugging.ddstate.logger.DDLogger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

/**
 * The test runner job executes the given test runner with the given test
 * description as eclipse job. You can use this job to run multiple test runs
 * parallel.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class TestRunnerJob extends Job {

	/** Personal logger for this class */
	final static DDLogger logger = DDStateLaunchPlugin
			.getLogger(TestRunnerJob.class);

	// status codes
	public final static int STATUS_CODE_OK = 0;

	public final static int STATUS_CODE_UNEXPECTED_ERROR = 1;

	public final static int STATUS_CODE_UNEXPECTED_TEST_RESULT = 2;

	private final IExtTestRunner fTestRunner;

	private final IJUnitTestDescription fTestDesc;

	private IPredicate fTestPredicate = null;

	// this variables are used in the post launch check
	private boolean fPostLaunchCheckIntalled = false;

	private int fExpectedResult = -1;

	/**
	 * Creates a new test runner job with the specified name. The job name is a
	 * human-readable value that is displayed to users. The name does not need
	 * to be unique, but it must not be null. The given test runner will run the
	 * given test description if the job is scheduled by the eclipse workbench.
	 * 
	 * @param jobName
	 *            the name of the jbo
	 * @param testRunner
	 *            the test runner to execute if the job is scheduled
	 * @param testDesc
	 *            the test description that should executed
	 */
	public TestRunnerJob(String jobName, IExtTestRunner testRunner,
			IJUnitTestDescription testDesc) {
		super(jobName);
		this.fTestRunner = testRunner;
		this.fTestDesc = testDesc;
	}

	/**
	 * Install a post launch check. This checks the result of the test run
	 * against the given expected test result.
	 * 
	 * @param expectedResult
	 *            the test result to check against the computed result
	 */
	public void installPostLaunchCheck(int expectedResult) {
		this.fPostLaunchCheckIntalled = true;
		this.fExpectedResult = expectedResult;
	}

	/**
	 * Check if the given result is equal to the expected result. If they are
	 * equal it's ok and the function returns <code>Status.OK_STATUS</code>.
	 * Otherwise the function returns an info status with
	 * <code>STATUS_CODE_UNEXPECTED_TEST_RESULT</code> as status code.
	 * 
	 * @param computedResult
	 *            the computed result by the junit tester.
	 * @return <code>Status.OK_STATUS</code> if the computed and the expected
	 *         result are equal, otherwise return an info status with
	 *         <code>STATUS_CODE_UNEXPECTED_TEST_RESULT</code> as status code
	 */
	private IStatus postLaunchCheck(int computedResult) {
		IStatus result = Status.OK_STATUS;

		if (this.fPostLaunchCheckIntalled) {
			if ((computedResult & this.fExpectedResult) == 0) {
				result = logger.log(IStatus.INFO,
						STATUS_CODE_UNEXPECTED_TEST_RESULT,
						"Computed result was '" + computedResult
								+ "', expected was '" + this.fExpectedResult
								+ "'", null);
			}
		}

		return result;
	}

	/**
	 * Excecute the test runner with the test description and call the post
	 * launch check if it was already installed. Returns a status. Look at the
	 * status code to interpret the result of this method. Following status
	 * results are possible. <br>
	 * <ul>
	 * <li><code>STATUS_CODE_OK</code>- the function returns without any
	 * errors. The post launch check was either disabled or succesful.</li>
	 * <li><code>STATUS_CODE_UNEXPECTED_ERROR</code>- an error occured on
	 * the test excecution. For more information look the exception included in
	 * the status object.</li>
	 * <li><code>STATUS_CODE_UNEXPECTED_TEST_RESULT</code>- the test run
	 * excecution have no errors but the post launch check was enabled and
	 * failed.</li>
	 * </ul>
	 * 
	 * @see org.eclipse.core.runtime.jobs.Job#run(org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected IStatus run(IProgressMonitor monitor) {
		IStatus result = Status.OK_STATUS;

		try {

			this.fTestPredicate = this.fTestRunner.run(this.fTestDesc);
			 if (this.fPostLaunchCheckIntalled) {
				result = this.postLaunchCheck(this.fTestPredicate.getState());
			}
			
		} catch (TestRunnerException e) {
			result = logger.log(IStatus.ERROR, STATUS_CODE_UNEXPECTED_ERROR,
					"Unexpected exception on testrunner execution", e);

		}

		return result;
	}

	/**
	 * Returns the test predicate if the test run was executed, otherwise
	 * <code>null</code>.
	 * 
	 * @return the test predicate if the test run was executed, otherwise
	 *         <code>null</code>.
	 */
	public IPredicate getTestPredicate() {
		return this.fTestPredicate;
	}
}