/*
 * $Id: DDRunResult.java,v 1.13 2005/08/19 15:18:34 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 19.04.2005 11:11:32 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.ddrun;

import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.Vector;

import org.deltadebugging.ddcore.DeltaSet;
import org.eclipse.core.runtime.IStatus;

/**
 * The DDRunResult is the result of one delta debugging run. But it does not
 * only contains the result by itself. The result holds informations about start
 * and stop time, computed delta set, passing and failing test description, the
 * test history and many more.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class DDRunResult implements IDDRunResult {

	private final static SimpleDateFormat DATE_FORMAT_NAME = new SimpleDateFormat(
			"dd.MM.yyyy HH:mm:ss");

	private int fState = IDDRunResult.STATE_SUCCESSFULL;

	private DeltaSet fComputedDeltaSet = new DeltaSet();

	private DeltaSet fMinimizedDeltaSet = new DeltaSet();

	private Date fStopTime = null;

	private final IDDRunDescription fDDRunDesc;

	private final Date fStartTime;

	private final Collection fStatus = new Vector();

	private final Collection fDDTestResults = new Vector();

	public DDRunResult(IDDRunDescription ddRunDesc, Date startTime) {
		this.fDDRunDesc = ddRunDesc;
		this.fStartTime = startTime;
	}

	public DDRunResult(IDDRunDescription ddRunDesc) {
		this(ddRunDesc, new Date(System.currentTimeMillis()));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#setComputedDeltaSet(org.deltadebugging.ddcore.DeltaSet)
	 */
	public void setComputedDeltaSet(DeltaSet computedDeltaSet) {
		this.fComputedDeltaSet = computedDeltaSet;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#setMinimizedDeltaSet(org.deltadebugging.ddcore.DeltaSet)
	 */
	public void setMinimizedDeltaSet(DeltaSet minimizedDeltaSet) {
		this.fMinimizedDeltaSet = minimizedDeltaSet;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#getComputedDeltaSet()
	 */
	public DeltaSet getComputedDeltaSet() {
		return this.fComputedDeltaSet;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#getMinimizedDeltaSet()
	 */
	public DeltaSet getMinimizedDeltaSet() {
		return this.fMinimizedDeltaSet;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#getStartTime()
	 */
	public Date getStartTime() {
		return this.fStartTime;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#getName()
	 */
	public String getName() {
		String result = null;

		String testName = null;
		if (this.fDDRunDesc.getPassTestDescription() != null) {
			testName = this.fDDRunDesc.getPassTestDescription()
					.getJUnitTestDescription().getTestCaseClassName();
		}

		result = this.fDDRunDesc.getDDRunID() + ") " + testName + " ("
				+ DATE_FORMAT_NAME.format(this.fStartTime) + ")";

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#getState()
	 */
	public int getState() {
		return this.fState;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#addTestResult(org.deltadebugging.ddstate.launcher.ddrun.IDDTestResult)
	 */
	public void addTestResult(IDDTestResult test) {
		this.fDDTestResults.add(test);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#getTestResults()
	 */
	public IDDTestResult[] getTestResults() {
		return (IDDTestResult[]) this.fDDTestResults
				.toArray(new IDDTestResult[this.fDDTestResults.size()]);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#getStopTime()
	 */
	public Date getStopTime() {
		return this.fStopTime;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#setStopTime(java.util.Date)
	 */
	public void setStopTime(Date stopTime) {
		this.fStopTime = stopTime;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#getDDRunDescription()
	 */
	public IDDRunDescription getDDRunDescription() {
		return this.fDDRunDesc;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#addStatus(org.eclipse.core.runtime.IStatus)
	 */
	public void addStatus(IStatus status) {
		this.fStatus.add(status);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#hasErrorStatus()
	 */
	public boolean hasErrorStatus() {
		boolean result = false;

		Iterator itStatus = this.fStatus.iterator();
		while (itStatus.hasNext() && result == false) {
			IStatus status = (IStatus) itStatus.next();

			if (status.getSeverity() == IStatus.ERROR) {
				result = true;
			}
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#hasInfoStatus()
	 */
	public boolean hasInfoStatus() {
		boolean result = false;

		Iterator itStatus = this.fStatus.iterator();
		while (itStatus.hasNext() && result == false) {
			IStatus status = (IStatus) itStatus.next();

			if (status.getSeverity() == IStatus.INFO) {
				result = true;
			}
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#getErrorStatus()
	 */
	public Collection getErrorStatus() {
		Collection result = new Vector();

		Iterator itStatus = this.fStatus.iterator();
		while (itStatus.hasNext()) {
			IStatus status = (IStatus) itStatus.next();

			if (status.getSeverity() == IStatus.ERROR) {
				result.add(status);
			}
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#getInfoStatus()
	 */
	public Collection getInfoStatus() {
		Collection result = new Vector();

		Iterator itStatus = this.fStatus.iterator();
		while (itStatus.hasNext()) {
			IStatus status = (IStatus) itStatus.next();

			if (status.getSeverity() == IStatus.INFO) {
				result.add(status);
			}
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#getStatus()
	 */
	public Collection getStatus() {
		return this.fStatus;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult#hasStatus(int)
	 */
	public boolean hasStatus(int statusCode) {
		boolean result = false;

		Iterator itStatus = this.fStatus.iterator();
		while (itStatus.hasNext() && result == false) {
			IStatus status = (IStatus) itStatus.next();

			if (status.getCode() == statusCode) {
				result = true;		}
		}

		return result;
	}
}