/*
 * $Id: DeltaComparator.java,v 1.6 2005/11/21 19:09:02 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 28.08.2005 13:53:37 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.tester;

import java.util.Comparator;

import org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta;
import org.deltadebugging.ddstate.memorygraph.delta.create.CreateObjectDelta;
import org.deltadebugging.ddstate.memorygraph.delta.create.CreatePrimitiveValueDelta;
import org.deltadebugging.ddstate.memorygraph.delta.delete.DeleteObjectReferenceDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdateArraySizeDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdateObjectReferenceDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdatePrimitiveValueDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdateStringDelta;

/**
 * Compare two deltas. This comparator should used in addition wiht a SortedSet
 * to sort the deltas in the way they can applied to the memory. All
 * CreateObjectDelta's should call at first, all UpdateObjectReference deltas at
 * last. Between this two groups of deltas the rest of the deltas will ordered
 * by their dependencies.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann</a>
 */
class DeltaComparator implements Comparator {

	private static DeltaComparator instance = null;

	/**
	 * Give every delta a position and so a defined order.
	 */
	private final static Integer CREATE_OBJECT_POS = new Integer(
			Integer.MIN_VALUE);
	private final static Integer UPDATE_ARRAY_SIZE_POS = new Integer(1);
	private final static Integer CREATE_PRIMITIVE_VALUE_POS = new Integer(2);
	private final static Integer UPDATE_STRING_POS = new Integer(10);
	private final static Integer UPDATE_PRIMITIVE_VALUE_POS = new Integer(10);
	private final static Integer UNKNOWN_DELTA_POS = new Integer(15);
	private final static Integer DELETE_OBJECT_REFERENCE_POS = new Integer(20);
	private final static Integer UPDATE_OBJECT_REFERENCE_POS = new Integer(
			Integer.MAX_VALUE);

	private DeltaComparator() {
		// use getInstance instead
	}

	public static DeltaComparator getInstance() {
		if (instance == null) {
			instance = new DeltaComparator();
		}

		return instance;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
	 */
	public int compare(Object o1, Object o2) {
		int result = 0;

		if (!(o1 instanceof IMemoryGraphDelta)) {
			throw new ClassCastException(
					"Given first argument is not an instance of IMemoryGraphDelta");
		}

		if (!(o2 instanceof IMemoryGraphDelta)) {
			throw new ClassCastException(
					"Given second argument is not an instance of IMemoryGraphDelta");
		}

		if (o1 == o2) {
			result = 0;
		} else {

			IMemoryGraphDelta delta1 = (IMemoryGraphDelta) o1;
			IMemoryGraphDelta delta2 = (IMemoryGraphDelta) o2;
			Integer pos1 = this.getPositionFor(delta1);
			Integer pos2 = this.getPositionFor(delta2);

			if (pos1.equals(pos2)) {
				result = new Integer(delta1.hashCode()).compareTo(new Integer(
						delta2.hashCode()));
			} else {
				result = pos1.compareTo(pos2);
			}
		}

		return result;
	}

	/**
	 * Returns the position for the delta.
	 * 
	 * @param delta
	 * @return the position for the delta.
	 */
	private Integer getPositionFor(IMemoryGraphDelta delta) {
		if (delta instanceof CreateObjectDelta)
			return CREATE_OBJECT_POS;
		
		if (delta instanceof UpdateArraySizeDelta)
			return UPDATE_ARRAY_SIZE_POS;

		if (delta instanceof CreatePrimitiveValueDelta)
			return CREATE_PRIMITIVE_VALUE_POS;

		if (delta instanceof UpdateStringDelta)
			return UPDATE_STRING_POS;

		if (delta instanceof UpdatePrimitiveValueDelta)
			return UPDATE_PRIMITIVE_VALUE_POS;

		if (delta instanceof DeleteObjectReferenceDelta)
			return DELETE_OBJECT_REFERENCE_POS;

		if (delta instanceof UpdateObjectReferenceDelta)
			return UPDATE_OBJECT_REFERENCE_POS;

		return UNKNOWN_DELTA_POS;
	}

}