/*
 * $Id: DeltaProcessor.java,v 1.19 2006/01/24 18:32:27 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 02.06.2005 14:44:02 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.tester;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import org.deltadebugging.ddcore.DeltaSet;
import org.deltadebugging.ddstate.launcher.BreakpointHitListenerAdapter;
import org.deltadebugging.ddstate.launcher.DDStateLaunchPlugin;
import org.deltadebugging.ddstate.launcher.client.ITestRunner;
import org.deltadebugging.ddstate.logger.DDLogger;
import org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta;
import org.deltadebugging.ddstate.memorygraph.delta.create.CreateObjectDelta;
import org.deltadebugging.ddstate.memorygraph.delta.create.CreatePrimitiveValueDelta;
import org.deltadebugging.ddstate.memorygraph.delta.delete.DeleteObjectReferenceDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdateArraySizeDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdateObjectReferenceDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdatePrimitiveValueDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdateStringDelta;
import org.deltadebugging.ddstate.memorygraph.exceptions.DeltaExecutionException;
import org.eclipse.jdt.debug.core.IJavaBreakpoint;
import org.eclipse.jdt.debug.core.IJavaBreakpointListener;
import org.eclipse.jdt.debug.core.IJavaThread;

/**
 * This processor is also an breakpoint listener. If the given breakpoint is hit
 * this processor apply the given delta set using the testrunner and the current
 * suspended thread.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class DeltaProcessor extends BreakpointHitListenerAdapter {

    /** Personal logger for this class */
    final static DDLogger logger = DDStateLaunchPlugin
            .getLogger(DeltaProcessor.class);

    private DeltaSet fDeltaSet;

    final ITestRunner fTestRunner;

    private int fCounter = 0;

    public DeltaProcessor(ITestRunner testRunner, IJavaBreakpoint bp) {
        super(bp);
        this.fTestRunner = testRunner;
    }

    public DeltaProcessor(ITestRunner testRunner) {
        super(new IJavaBreakpoint[0]);
        this.fTestRunner = testRunner;
    }

    /**
     * Set the delta set that should applied to the memory graph.
     * 
     * @param deltaSet
     */
    public void setTestSet(DeltaSet deltaSet) {
        this.fDeltaSet = deltaSet;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.launcher.listener.BreakpointHitListenerAdapter#givenBreakpointHit(org.eclipse.jdt.debug.core.IJavaThread,
     *      org.eclipse.jdt.debug.core.IJavaBreakpoint)
     */
    protected int givenBreakpointHit(IJavaThread thread,
            IJavaBreakpoint breakpoint) {

        // BugFix: remove the breakpoint before we process the deltas. This is
        // required because it happens that the installed breakpoint stopped the
        // processing of the deltas (on creating new empty class)
        this.fTestRunner.getDebugTarget().breakpointRemoved(breakpoint, null);

        try {
            this.process(thread);
            this.disable(IJavaBreakpointListener.DONT_SUSPEND);

        } finally {

            // After all we have to ensure that the breakpoint will installed
            this.fTestRunner.getDebugTarget().breakpointAdded(breakpoint);
        }

        return IJavaBreakpointListener.DONT_SUSPEND;
    }

    public void process(final IJavaThread thread) {
        // reset counter
        this.fCounter = 1;

        // reset the errors
        this.setError(null);

        SplittedDeltaSet splittedDeltas = this.splitDeltaSet(this.fDeltaSet);

        // apply CreateObjectDeltas
        this.applyDeltas(splittedDeltas.colCreateObjectDeltas, thread);

        // apply array deltas
        this.applyDeltas(splittedDeltas.colIncreaseArraySizeDeltas, thread);
        this.applyDeltas(splittedDeltas.colCreatePrimitiveValueDeltas, thread);

        // update string and primitive values
        this.applyDeltas(splittedDeltas.colUpdateStringDeltas, thread);
        this.applyDeltas(splittedDeltas.colUpdatePrimitiveValueDeltas, thread);

        // delete object references
        this.applyDeltas(splittedDeltas.colDecreaseArraySizeDeltas, thread);
        this.applyDeltas(splittedDeltas.colDeleteObjectReferenceDeltas, thread);

        // update object references
        this.iterateOverDeltas(splittedDeltas.colUpdateObjectReferenceDeltas,
                new IDeltaSelector() {
                    public void deltaSelected(IMemoryGraphDelta delta)
                            throws DeltaExecutionException {
                        ((UpdateObjectReferenceDelta) delta).prepare(thread,
                                DeltaProcessor.this.fTestRunner);
                    }
                });
        this.applyDeltas(splittedDeltas.colUpdateObjectReferenceDeltas, thread);
    }

    /**
     * Splitt the the given delta set in the different classes of deltas and
     * return a splitted delta set.
     * 
     * @param deltaSet
     * @return a splitted delta set.
     */
    private SplittedDeltaSet splitDeltaSet(DeltaSet deltaSet) {
        SplittedDeltaSet result = new SplittedDeltaSet();

        Iterator itDeltaSet = deltaSet.iterator();

        while (itDeltaSet.hasNext()) {
            IMemoryGraphDelta delta = (IMemoryGraphDelta) itDeltaSet.next();

            if (delta instanceof CreateObjectDelta) {
                result.colCreateObjectDeltas.add(delta);

            } else if (delta instanceof CreatePrimitiveValueDelta) {
                result.colCreatePrimitiveValueDeltas.add(delta);

            } else if (delta instanceof UpdateObjectReferenceDelta) {
                result.colUpdateObjectReferenceDeltas.add(delta);

            } else if (delta instanceof UpdateStringDelta) {
                result.colUpdateStringDeltas.add(delta);

            } else if (delta instanceof UpdateArraySizeDelta) {
                UpdateArraySizeDelta arraySizeDelta = (UpdateArraySizeDelta) delta;

                if (arraySizeDelta.getOldArrayLength() < arraySizeDelta
                        .getNewArrayLength())
                    result.colIncreaseArraySizeDeltas.add(delta);

                else
                    result.colDecreaseArraySizeDeltas.add(delta);

            } else if (delta instanceof UpdatePrimitiveValueDelta) {
                result.colUpdatePrimitiveValueDeltas.add(delta);

            } else if (delta instanceof DeleteObjectReferenceDelta) {
                result.colDeleteObjectReferenceDeltas.add(delta);

            } else {
                throw new RuntimeException("Unknown delta");
            }

        }

        return result;
    }
    /**
     * The splitted delta set is the container of collections of deltas. For
     * every available delta class it exists an own collection.
     * 
     * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann</a>
     */
    class SplittedDeltaSet {

        public Collection colCreateObjectDeltas = new Vector();

        public Collection colCreatePrimitiveValueDeltas = new Vector();

        public Collection colUpdateObjectReferenceDeltas = new Vector();

        public Collection colUpdateStringDeltas = new Vector();

        public Collection colIncreaseArraySizeDeltas = new Vector();

        public Collection colDecreaseArraySizeDeltas = new Vector();

        public Collection colUpdatePrimitiveValueDeltas = new Vector();

        public Collection colDeleteObjectReferenceDeltas = new Vector();

    }

    /**
     * Iterate over all deltas in the given delta set and call
     * <code>delta.apply(thread, DeltaProcessor.this.fTestRunner</code> for
     * every delta.
     * 
     * @param deltas
     * @param thread
     * @see IMemoryGraphDelta#apply(IJavaThread, ITestRunner)
     */
    private void applyDeltas(Collection deltas, final IJavaThread thread) {
        this.iterateOverDeltas(deltas, new IDeltaSelector() {
            public void deltaSelected(IMemoryGraphDelta delta)
                    throws DeltaExecutionException {
                DeltaProcessor.this.traceApply(delta);
                
                delta.apply(thread, DeltaProcessor.this.fTestRunner);
            }
        });
    }

    void traceApply(IMemoryGraphDelta delta) {
        String message = this.fCounter++ + " / " + this.fDeltaSet.size()
                + " - " + delta.toString();

        DeltaProcessor.logger.trace(DDStateLaunchPlugin.DEBUG_DELTA_PROCESSOR,
                message);
    }

    /**
     * Iterate over all deltas in the given delta set and call for every dela
     * <code>selector.deltaSelect()</code> with the delta as argument. The
     * iteration stops, if an error occured.
     * 
     * @param deltas
     * @param selector
     *            the selector will called for every delta in the delta set.
     */
    private void iterateOverDeltas(Collection deltas, IDeltaSelector selector) {
        Iterator itDeltas = deltas.iterator();

        while (itDeltas.hasNext() && !this.hasErros()) {
            Object element = itDeltas.next();

            if (element instanceof IMemoryGraphDelta) {
                try {
                    selector.deltaSelected((IMemoryGraphDelta) element);

                } catch (Throwable t) {
                    DeltaProcessor.logger.trace(
                            DDStateLaunchPlugin.DEBUG_DELTA_PROCESSOR,
                            "Delta proccessing failed", t);
                    this.setError(t);
                }
            }
        }
    }

    interface IDeltaSelector {
        public void deltaSelected(IMemoryGraphDelta delta)
                throws DeltaExecutionException;
    }
}
