/*
 * $Id: GraphDecorator.java,v 1.13 2005/11/28 11:12:49 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 03.01.2005 09:52:44 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.comparator.delta;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.deltadebugging.ddstate.DDStatePlugin;
import org.deltadebugging.ddstate.comparator.exceptions.DeltaComparatorException;
import org.deltadebugging.ddstate.logger.DDLogger;
import org.deltadebugging.ddstate.memorygraph.IGraphEdge;
import org.deltadebugging.ddstate.memorygraph.IGraphElement;
import org.deltadebugging.ddstate.memorygraph.IGraphNode;
import org.deltadebugging.ddstate.memorygraph.value.IArrayNode;
import org.deltadebugging.ddstate.memorygraph.value.IPrimitiveValueNode;
import org.deltadebugging.ddstate.memorygraph.value.IStringObjectNode;

/**
 * COMMENTME Comment type GraphDecorator
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class GraphDecorator {

    // Personal logger for this class
    private final static DDLogger logger = DDStatePlugin
            .getLogger(GraphDecorator.class);

    public final static byte NOT_COMPARED = 1;
    public final static byte EQUAL = 2;
    public final static byte ONLY_IN_FIRSTGRAPH = 4;
    public final static byte ONLY_IN_SECONDGRAPH = 8;
    public final static byte DIFFERENT = 16;

    private static Map fGraphDecoratorListeners = new HashMap();

    private static Iterator getGraphDecoratorListeners() {
        return GraphDecorator.fGraphDecoratorListeners.keySet().iterator();
    }

    public static void addGraphDecoratorListener(
            IGraphDecoratorListener listener) {
        if (!GraphDecorator.fGraphDecoratorListeners.containsKey(listener)) {
            GraphDecorator.fGraphDecoratorListeners.put(listener, null);
        }
    }

    public static void removeGraphDecoratorListener(
            IGraphDecoratorListener listener) {
        if (!GraphDecorator.fGraphDecoratorListeners.containsKey(listener)) {
            GraphDecorator.fGraphDecoratorListeners.remove(listener);
        }
    }

    // FIXME Listener Konzept ?berdenken
    public static void notifiyListeners(IGraphElement graphElement, byte mask) {
        for (Iterator itListeners = getGraphDecoratorListeners(); itListeners
                .hasNext();) {
            IGraphDecoratorListener listener = (IGraphDecoratorListener) itListeners
                    .next();
            listener.graphElementMaskAs(graphElement, mask);
        }
    }

    /**
     * Decorate the given graph element with the given mask.
     * 
     * <b>Note: </b> The graph element has to masked as
     * <code>NOT_COMPARED</code> and the mask is one of the following list
     * <code>[EQUAL, ONLY_IN_FIRSTGRAPH, ONLY_IN_SECONDGRAPH, DIFFERENT</code>.
     * 
     * @param graphElement
     *            the graph element to mask
     * @param mask
     */
    private static void maskGraphElementAs(IGraphElement graphElement, byte mask) {
        assert graphElement.isMaskedAs(NOT_COMPARED);
        assert mask == EQUAL || mask == ONLY_IN_FIRSTGRAPH
                || mask == ONLY_IN_SECONDGRAPH || mask == DIFFERENT;

        graphElement.maskAs(mask);
        notifiyListeners(graphElement, mask);
    }

    /**
     * Decorate all the elements of the given iterator with the given mask.
     * 
     * @param itGraphElements
     *            iterator of graph elements.
     * @param mask
     * @throws DeltaComparatorException
     *             if at leat one element of the iterator is not an intance of
     *             <code>IGraphElement</code>.
     */
    public static void decorateGraphElementsAs(Iterator itGraphElements,
    		byte mask) throws DeltaComparatorException {

        IGraphElement graphElement = null;
        Object nextObject = null;

        while (itGraphElements.hasNext()) {

            nextObject = itGraphElements.next();

            if (!(nextObject instanceof IGraphElement)) {
                throw new DeltaComparatorException("Giving object '"
                        + nextObject.getClass().getName()
                        + " is not supported for this function.");
            }

            graphElement = (IGraphElement) nextObject;

            if (graphElement instanceof IGraphEdge) {
                GraphDecorator.decorateGraphEdgeAs((IGraphEdge) graphElement,
                        mask, true);

            } else if (graphElement instanceof IGraphNode) {
                GraphDecorator.decorateGraphNodeAs((IGraphNode) graphElement,
                        mask);
            }

        }
    }

    /**
     * Decorate the given two graph elements as <code>EQUAL</code>.
     * 
     * <b>Note: </b>
     * <code>firstElement.graphEquals(secondElement) == true</code>,
     * <code>firstElement.getAssociatedElement() == secondElement</code> and
     * <code>secondElement.getAssociatedElement() == firstElement</code>.
     * 
     * @param firstElement
     *            first graph element to mask as equal.
     * @param secondElement
     *            second graph element to mask as equal.
     */
    public static void decorateGraphElementsAsEqual(IGraphElement firstElement,
            IGraphElement secondElement) {

        assert firstElement.graphEquals(secondElement) == true;
        assert firstElement.isAssociatedWith(secondElement);

        GraphDecorator.maskGraphElementAs(firstElement, EQUAL);
        GraphDecorator.maskGraphElementAs(secondElement, EQUAL);
    }

    public static void decorateGraphElementsAsDifferent(
            IGraphElement firstElement, IGraphElement secondElement) {

        assert (firstElement instanceof IPrimitiveValueNode && secondElement instanceof IPrimitiveValueNode)
                || (firstElement instanceof IStringObjectNode && secondElement instanceof IStringObjectNode)
                || (firstElement instanceof IGraphEdge && secondElement instanceof IGraphEdge)
                || (firstElement instanceof IArrayNode && secondElement instanceof IArrayNode);

        if (firstElement instanceof IGraphEdge) {
            GraphDecorator.decorateGraphEdgeAs((IGraphEdge) firstElement,
                    ONLY_IN_FIRSTGRAPH, true);
            GraphDecorator.decorateGraphEdgeAs((IGraphEdge) secondElement,
                    ONLY_IN_SECONDGRAPH, true);

        } else {

            GraphDecorator.maskGraphElementAs(firstElement, DIFFERENT);
            GraphDecorator.maskGraphElementAs(secondElement, DIFFERENT);
        }
    }

    /**
     * Decorate the given graph edge with the given mask if it is masked as
     * <code>NOT_COMPARED</code>. The given mask will propagate to its target
     * node if and only if the propagate flag is set.
     * 
     * <b>Note: </b> If the graph edge has to decorate as
     * <code>ONLY_IN_FIRSTGRAPH</code> or <code>ONLY_IN_SECONDGRAPH</code>
     * the edge must not associated to another edge.
     * 
     * @param edge
     *            the graph edge to decorate.
     * @param mask
     *            <code>ONLY_IN_FIRSTGRAPH</code> or
     *            <code>ONLY_IN_SECONDGRAPH</code>
     * @param propagate
     *            flag if the mask should propagate to the target node of the
     *            edge.
     */
    public static void decorateGraphEdgeAs(IGraphEdge edge, byte mask,
            boolean propagate) {

        assert (mask == ONLY_IN_FIRSTGRAPH) || (mask == ONLY_IN_SECONDGRAPH);

        if (edge.isMaskedAs(NOT_COMPARED)) {
            GraphDecorator.maskGraphElementAs(edge, mask);

            if (propagate) {
                IGraphNode targetNode = edge.getTargetNode();

                if (targetNode.isMaskedAs(NOT_COMPARED)) {
                    if (!targetNode.hasAssociatedElement()) {
                        GraphDecorator.decorateGraphNodeAs(targetNode, mask);

                    }

                    // Outcomment (2005-10-01): if the target node has an
                    // associated element we do nothing, because it will
                    // compared later (together with his associated element).
                    // else {
                    // GraphDecorator.decorateGraphElementsAsEqual(targetNode,
                    // targetNode.getAssociatedElement());
                    // }
                }
            }

        } else {
            logger.trace(DDStatePlugin.DEBUG_GRAPH_COMPARATOR_DELTACOMPARATOR,
                    "Ignore command to mask edge " + edge + "(mask = "
                            + edge.getDifferenceMask() + ") with mask " + mask);
        }
    }

    /**
     * Decorate the given graph node with the given mask if it is masked as
     * <code>NOT_COMPARED</code>. If the given node exits only in the first
     * or in the second graph the given mask will propagate to his successors
     * (all successor edges).
     * 
     * <b>Note: </b> If the graph node has to decorate as
     * <code>ONLY_IN_FIRSTGRAPH</code> or <code>ONLY_IN_SECONDGRAPH</code>
     * the node must not associated to another node. If the graph node has to
     * decorate as <code>DIFFERNET</code> the node must an instance of
     * <code>IPrimitiveValueNode</code>.
     * 
     * @param node
     *            the graph node to decorate.
     * @param mask
     *            <code>ONLY_IN_FIRSTGRAPH</code>,
     *            <code>ONLY_IN_SECONDGRAPH</code>
     */
    public static void decorateGraphNodeAs(IGraphNode node, byte mask) {
        assert (mask == ONLY_IN_FIRSTGRAPH && !node.hasAssociatedElement())
                || (mask == ONLY_IN_SECONDGRAPH && !node.hasAssociatedElement());

        if (node.isMaskedAs(NOT_COMPARED)) {
            GraphDecorator.maskGraphElementAs(node, mask);

            if (mask == ONLY_IN_FIRSTGRAPH || mask == ONLY_IN_SECONDGRAPH) {
                Iterator itSuccessors = node.getChildren().iterator();
                IGraphEdge successors = null;

                while (itSuccessors.hasNext()) {
                    successors = (IGraphEdge) itSuccessors.next();
                    GraphDecorator.decorateGraphEdgeAs(successors, mask, true);
                }
            }

        } else {
            logger.trace(DDStatePlugin.DEBUG_GRAPH_COMPARATOR_DELTACOMPARATOR,
                    "Ignore command to mask node " + node + "(mask = "
                            + node.getDifferenceMask() + ") with mask " + mask);
        }
    }
}