/*
 * $Id: GraphEdgeDiff.java,v 1.2 2005/08/16 14:41:52 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 21.04.2005 12:19:29 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.comparator.delta;

import java.util.Collection;
import java.util.Vector;

import org.deltadebugging.ddstate.memorygraph.IGraphEdge;

/**
 * The graph edge diff is a container to collect the result of the <code>GraphEdgeDiffUtil</code>.
 * 
 * @see org.deltadebugging.ddstate.comparator.delta.GraphEdgeDiffUtil
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class GraphEdgeDiff {

	/** Collection of graph edges only available in the first graph */
	private Collection fEdgesOnlyInFirstGraph = null;

	/** Collection of graph edges only available in the second graph */
	private Collection fEdgesOnlyInSecondGraph = null;

	/**
	 * Collection of graph edges pairs which are available in the first and in
	 * the second graph
	 */
	private Collection fEdgesInBothGraph = null;

	public GraphEdgeDiff() {
		this.fEdgesOnlyInFirstGraph = new Vector();
		this.fEdgesOnlyInSecondGraph = new Vector();
		this.fEdgesInBothGraph = new Vector();
	}

	/**
	 * Add the given edge to the edges only available in the first graph.
	 * 
	 * @param edge
	 *            one graph edge only existing in the first graph.
	 */
	public void addEdgeOnlyInFirstGraph(IGraphEdge edge) {
		this.fEdgesOnlyInFirstGraph.add(edge);
	}

	/**
	 * Add the given edges to the edges only available in the first graph.
	 * 
	 * @param edges
	 *            a collection of graph edge only existing in the first graph.
	 */
	public void addEdgesOnlyInFirstGraph(Collection edges) {
		this.fEdgesOnlyInFirstGraph.add(edges);
	}

	/**
	 * Add the given edge to the edges only available in the second graph.
	 * 
	 * @param edge
	 *            one graph edge only existing in the second graph.
	 */
	public void addEdgeOnlyInSecondGraph(IGraphEdge edge) {
		this.fEdgesOnlyInSecondGraph.add(edge);
	}

	/**
	 * Add the given edges to the edges only available in the second graph.
	 * 
	 * @param edges
	 *            a collection of graph edge only existing in the second graph.
	 */
	public void addEdgesOnlyInSecondGraph(Collection edges) {
		this.fEdgesOnlyInSecondGraph.add(edges);
	}

	/**
	 * Add the given edge from the first and the second graph as graph edge pair
	 * to the collection of graph edge pairs.
	 * 
	 * @param firstEdge
	 *            edge existing in the first graph.
	 * @param secondEdge
	 *            edge existing in the second graph.
	 */
	public void addEdgesInBothGraph(IGraphEdge firstEdge, IGraphEdge secondEdge) {
		GraphEdgePair pair = new GraphEdgePair(firstEdge, secondEdge);
		this.fEdgesInBothGraph.add(pair);
	}

	/**
	 * Returns a collection of graph edge pairs with edges that exist in the
	 * first and in the second graph.
	 * 
	 * @return a collection of graph edge pairs with edges that exist in the
	 *         first and in the second graph.
	 */
	public Collection getEdgesInBothGraph() {
		return this.fEdgesInBothGraph;
	}

	/**
	 * Returns a collection of edges that only exist in the first graph.
	 * 
	 * @return a collection of edges that only exist in the first graph.
	 */
	public Collection getEdgesOnlyInFirstGraph() {
		return this.fEdgesOnlyInFirstGraph;
	}

	/**
	 * Returns a collection of edges that only exist in the second graph.
	 * 
	 * @return a collection of edges that only exist in the second graph.
	 */
	public Collection getEdgesOnlyInSecondGraph() {
		return this.fEdgesOnlyInSecondGraph;
	}
}