/*
 * $Id: GraphExtractor.java,v 1.32 2005/11/20 14:24:16 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 20.09.2004 12:44:45 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.extractor;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import org.deltadebugging.ddstate.DDStatePlugin;
import org.deltadebugging.ddstate.IDDStatePreferenceConstants;
import org.deltadebugging.ddstate.extractor.exceptions.GraphExtractorException;
import org.deltadebugging.ddstate.extractor.exceptions.MemoryAccessException;
import org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.deltadebugging.ddstate.memorygraph.IStackFrame;
import org.deltadebugging.ddstate.memorygraph.RootNode;
import org.deltadebugging.ddstate.memorygraph.StackFrame;
import org.deltadebugging.ddstate.memorygraph.exceptions.MemoryGraphExtractionException;
import org.deltadebugging.ddstate.memorygraph.filter.GraphFilterException;
import org.deltadebugging.ddstate.memorygraph.filter.GraphFilterManager;
import org.deltadebugging.ddstate.memorygraph.filter.IGraphFilter;
import org.deltadebugging.ddstate.memorygraph.filter.typefilter.JavaTypeNameFilter;
import org.deltadebugging.ddstate.memorygraph.util.MemoryGraphUtil;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.debug.core.DebugException;
import org.eclipse.jdt.core.Signature;
import org.eclipse.jdt.debug.core.IJavaReferenceType;
import org.eclipse.jdt.debug.core.IJavaStackFrame;
import org.eclipse.jdt.debug.core.IJavaThread;
import org.eclipse.jdt.debug.core.IJavaType;
import org.eclipse.jdt.debug.core.IJavaVariable;
import org.eclipse.jface.preference.IPreferenceStore;

/**
 * The graph extractor create a memory graph at a given position on the programm
 * execution. This means you run your program until your preferred position is
 * reached and suspend the thread. After that you can call the graph extractor.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class GraphExtractor extends BaseGraphExtractor {

	private GraphFilterManager fFilterManager = null;

	/**
	 * Set the variable to true causes a check that all nodes and edges that was
	 * created are reachable from the root node. Are not all nodes or edges
	 * reachable an IllegalStateException will thrown. The precondition for this
	 * check is that on graph extraction the created nodes and edges are count.
	 * Therefore the constant DEBUG_GRAPH_STATISTICS must true.
	 * 
	 * @see DDStatePlugin.DEBUG_GRAPH_STATISTICS
	 */
	private final static boolean ASSERT_CREATED_GRAPH_ELEMENTS = false && DDStatePlugin.DEBUG_GRAPH_STATISTICS;

	public GraphExtractor() {
		this.fFilterManager = new GraphFilterManager();
		this.initializeGraphFilter();
	}

	private void initializeGraphFilter() {
		IPreferenceStore preferences = DDStatePlugin.getDefault()
				.getPreferenceStore();

		// add the different graph filter to the manager
		String[] typeNames = this
				.parseList(preferences
						.getString(IDDStatePreferenceConstants.GRAPHEXTRACTOR_ACTIVE_FILTERS_LIST));
		IGraphFilter typeNameFilter = new JavaTypeNameFilter(typeNames);
		this.fFilterManager.addFilter(typeNameFilter);
	}

	/**
	 * Parses the comma separated string into an array of strings
	 * 
	 * @return list
	 */
	private String[] parseList(String listString) {
		List list = new ArrayList(10);
		StringTokenizer tokenizer = new StringTokenizer(listString, ","); //$NON-NLS-1$
		while (tokenizer.hasMoreTokens()) {
			String token = tokenizer.nextToken();
			list.add(token);
		}
		return (String[]) list.toArray(new String[list.size()]);
	}

	/**
	 * Create a new memory graph of the given thread. It is recommanded that the
	 * thread is suspended at the position where the memory graph should be
	 * extracted.
	 * 
	 * @param thread
	 * @param passingRun
	 *            set to true if you want to extract the graph of the passing
	 *            run, else false.
	 * @return a new memory graph.
	 * @throws GraphExtractorException
	 *             if the given thread is not suspended or an exception occured
	 *             on reading out the inforamtion of the memory.
	 * @throws IllegalArgumentException
	 *             if the given thread is null.
	 */
	public IRootNode createMemoryGraph(int ddRunID, final IJavaThread thread,
			boolean passingRun, IJUnitTestDescription testDesc,
			IProgressMonitor monitor) throws GraphExtractorException {

		if (thread == null)
			throw new IllegalArgumentException(
					"The given thread must not be null.");

		if (thread.isSuspended() == false)
			throw new GraphExtractorException(
					"The given thread must be suspended.");

		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}

		RootNode resultGraph = new RootNode(passingRun, ddRunID);
		
		try {
			monitor.beginTask("Extract memory graphs...", 100);

			monitor.subTask("Extract memory graphs (add base variables)...");
			// Add all base variables to the graph. This are all variables
			// accessible from all stackframe including the no-named (this)
			// variable.
			this.addLocalVariables(thread, resultGraph, testDesc);

			// Add all classes to the graph, which has static attributes
			this.addTypes(thread, resultGraph);
			monitor.worked(30);

			try {
				monitor
						.subTask("Extract memory graphs (unfold base objects)...");
				resultGraph.unfoldSubgraph(monitor);
				monitor.worked(70);
			} catch (MemoryGraphExtractionException e) {
				logger.log(IStatus.ERROR,
						"Failure on extracting and unfolding the memorygraph",
						e);

				throw new GraphExtractorException(
						"Failure on extracting and unfolding the memorygraph",
						e);
			}
		} finally {
			monitor.done();
		}

		// if the creation process was canceld return null
		if (monitor.isCanceled()) {
			return null;
		}

		this.notifyGraphExtracted(resultGraph);

		logger.trace(DDStatePlugin.DEBUG_GRAPH_STATISTICS, resultGraph
				.getStatistics().toString());

		if (ASSERT_CREATED_GRAPH_ELEMENTS) {
			this.checkCreateNumberOfGraphElements(resultGraph);
		}

		return resultGraph;
	}

	/**
	 * Check if the created graph elements are all reachable from the given root
	 * node.
	 * 
	 * @param rootNode
	 * @throws IllegalStateException
	 *             if not all graph elements are reachable from the given root
	 *             node.
	 */
	private void checkCreateNumberOfGraphElements(IRootNode rootNode) {
		if (rootNode == null)
			return;

		MemoryGraphUtil.CountResult count = MemoryGraphUtil
				.countNodeAndEdges(rootNode);

		if (rootNode.getStatistics().numberOfGraphEdges != count.numberOfEdges) {
			String msg = "Count " + count.numberOfEdges
					+ " number of edges - exptected "
					+ rootNode.getStatistics().numberOfGraphEdges;
			logger.log(IStatus.ERROR, msg);
			throw new IllegalStateException(msg);
		}

		if (rootNode.getStatistics().numberOfGraphNodes != count.numberOfNodes) {
			String msg = "Count " + count.numberOfNodes
					+ " number of nodes - exptected "
					+ rootNode.getStatistics().numberOfGraphNodes;
			logger.log(IStatus.ERROR, msg);
			throw new IllegalStateException(msg);
		}
	}

	private void addTypes(IJavaThread thread, RootNode rootNode)
			throws GraphExtractorException {

		IJavaType[] types = null;
		try {
			types = MemoryAccessUtil.getTypesWithStaticVariables(thread);
		} catch (MemoryAccessException e) {
			throw new GraphExtractorException(
					"Cannot retriev the relevant reference types from the given java thread.",
					e);
		}

		// first filter all types not added
		Collection filteredTypes = new Vector();
		for (int i = 0; i < types.length; i++) {
			IJavaType type = types[i];

			boolean filterType = true;
			try {
				filterType = this.fFilterManager.filterObject(type);
			} catch (GraphFilterException e) {
				throw new GraphExtractorException(
						"Problems on filtering reference type", e);
			}

			if (!filterType) {
				filteredTypes.add(type);
			}
		}

		Iterator itFilteredTypes = filteredTypes.iterator();
		while (itFilteredTypes.hasNext()) {
			IJavaReferenceType type = (IJavaReferenceType) itFilteredTypes
					.next();

			try {
				rootNode.addType(type);
			} catch (MemoryGraphExtractionException e) {
				throw new GraphExtractorException(
						"Cannot create new type successor for the root node", e);
			}

			try {
				logger.trace(DDStatePlugin.DEBUG_GRAPH_EXTRACTOR,
						"Add class TypeNode with static values "
								+ type.getName());
			} catch (DebugException e) {
				// do nothing
			}

		}

	}

	/**
	 * Extract all the variables of all the frames of the given thread and add
	 * them to the given graph. If one of the frames is not instance of
	 * <code>IJavaStackFrame</code> this will be logged in the eclipse error
	 * log.
	 * 
	 * <p>
	 * <b>Note: </b> The given graph and thread must not be null. Also the
	 * thread has to be suspended, this means that the call
	 * <code>thread.isSuspended()</code> must return <code>true</code>.
	 * 
	 * @param thread
	 *            Java thread.
	 * @param rootNode
	 *            the rootNode which the variables of the given thread should be
	 *            added.
	 * @throws GraphExtractorException
	 *             if something went wrong on creating the graph nodes for the
	 *             given java frame.
	 */
	private void addLocalVariables(IJavaThread thread, RootNode rootNode,
			IJUnitTestDescription testDesc) throws GraphExtractorException {
		assert rootNode != null;
		assert thread != null;
		assert thread.isSuspended();

		IJavaStackFrame[] frames = null;

		try {
			frames = MemoryAccessUtil.getAllStackframes(thread);
		} catch (DebugException e) {
			throw new GraphExtractorException(
					"Cannot retrieve the stack frame of the given thread.", e);
		}

		// int frameLength = frames.length - STACKFRAME_LOWER_LIMIT;

		// search start frame
		int startFrameIndex = frames.length - 1;

		if (testDesc != null) {
			for (int i = frames.length - 1; i >= 0; i--) {
				IJavaStackFrame frame = frames[i];

				try {
					String methodType = frame.getDeclaringTypeName();
					String methodName = frame.getMethodName();
					String methodSignature = frame.getSignature();
					int paramSignatureCount = Signature
							.getParameterCount(methodSignature);

					if (paramSignatureCount == 0
							&& methodType.equals(testDesc
									.getTestCaseClassName())
							&& methodName.equals(testDesc.getTestName())) {
						startFrameIndex = i;
						break;
					}
				} catch (DebugException e) {
					throw new GraphExtractorException(
							"Cannot retrieve method informations from the stackframe",
							e);
				}
			}
		}

		for (int i = startFrameIndex; i >= 0; i--) {
			IStackFrame frame = null;
			try {
				frame = new StackFrame(frames[i], i, startFrameIndex + 1);
			} catch (MemoryGraphExtractionException e) {
				throw new GraphExtractorException("Cannot create stackframe", e);
			}

			this.extractStackFrame(frame, rootNode);
		}
	}

	/**
	 * Extract all variables of the given frame and add them to the given graph.
	 * If the variable is not a instance of <code>IJavaVariable</code> this
	 * will be logged in the Eclipse error log.
	 * 
	 * <p>
	 * <b>Note: </b> The given frame and graph must not be null.
	 * 
	 * @param frame
	 * @param frameNumber
	 *            the number of the stackframe in the callstack. The frame on
	 *            top has the number 0.
	 * @param graph
	 *            the graph which the variables of the given frame should be
	 *            added.
	 * @throws DebugException
	 * @throws GraphExtractorException
	 *             if something went wrong on creating the graph nodes for the
	 *             given java frame.
	 */
	private void extractStackFrame(IStackFrame stackFrame, RootNode graph)
			throws GraphExtractorException {
		assert stackFrame != null;
		assert graph != null;

		IJavaVariable[] variables = null;

		try {
			variables = MemoryAccessUtil.getAllJavaVariables(stackFrame
					.getUnderlyingStackFrame());
		} catch (DebugException e) {
			throw new GraphExtractorException(
					"Cannot extract the variables of the given stack frame.", e);
		}

		for (int i = 0; i < variables.length; i++) {
			try {
				graph.addLocalVariable(variables[i], stackFrame);

			} catch (MemoryGraphExtractionException e) {
				throw new GraphExtractorException(
						"Cannot create new local variable successor for the root node",
						e);
			}
		}

	}

}