/*
 * $Id: GraphElementUtil.java,v 1.7 2006/02/05 16:05:47 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 15.03.2005 09:32:57 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph;

import org.deltadebugging.ddstate.memorygraph.exceptions.MemoryGraphExtractionException;
import org.deltadebugging.ddstate.memorygraph.type.GlobalVariableEdge;
import org.deltadebugging.ddstate.memorygraph.type.IGlobalVariableEdge;
import org.deltadebugging.ddstate.memorygraph.type.IReferenceTypeNode;
import org.deltadebugging.ddstate.memorygraph.type.ITypeEdge;
import org.deltadebugging.ddstate.memorygraph.type.ITypeNode;
import org.deltadebugging.ddstate.memorygraph.type.PrimitiveTypeNode;
import org.deltadebugging.ddstate.memorygraph.type.ReferenceTypeNode;
import org.deltadebugging.ddstate.memorygraph.type.TypeEdge;
import org.deltadebugging.ddstate.memorygraph.value.ArrayElementEdge;
import org.deltadebugging.ddstate.memorygraph.value.ArrayNode;
import org.deltadebugging.ddstate.memorygraph.value.ClassObjectNode;
import org.deltadebugging.ddstate.memorygraph.value.GraphNodePrimitiveArrayReference;
import org.deltadebugging.ddstate.memorygraph.value.IArrayElementEdge;
import org.deltadebugging.ddstate.memorygraph.value.IArrayNode;
import org.deltadebugging.ddstate.memorygraph.value.IObjectFieldEdge;
import org.deltadebugging.ddstate.memorygraph.value.IObjectNode;
import org.deltadebugging.ddstate.memorygraph.value.IValueNode;
import org.deltadebugging.ddstate.memorygraph.value.NullValueNode;
import org.deltadebugging.ddstate.memorygraph.value.ObjectFieldEdge;
import org.deltadebugging.ddstate.memorygraph.value.ObjectNode;
import org.deltadebugging.ddstate.memorygraph.value.PrimitiveValueNode;
import org.deltadebugging.ddstate.memorygraph.value.StringObjectNode;
import org.deltadebugging.ddstate.memorygraph.value.ValueNode;
import org.eclipse.debug.core.DebugException;
import org.eclipse.jdt.core.Signature;
import org.eclipse.jdt.debug.core.IJavaArray;
import org.eclipse.jdt.debug.core.IJavaClassObject;
import org.eclipse.jdt.debug.core.IJavaObject;
import org.eclipse.jdt.debug.core.IJavaPrimitiveValue;
import org.eclipse.jdt.debug.core.IJavaReferenceType;
import org.eclipse.jdt.debug.core.IJavaType;
import org.eclipse.jdt.debug.core.IJavaValue;
import org.eclipse.jdt.debug.core.IJavaVariable;

/**
 * COMMENTME Comment type GraphNodeFactory
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class GraphElementUtil {

    private GraphElementUtil() {
        // avoid constructing a new class with the default constructor
    }

    /**
     * Create a new directed graph edge.
     * 
     * @param sourceNode
     *            the source node where the edge points from.
     * @param targetNode
     *            the target node where the edge points to.
     * @param name
     *            the name of the edge. The name is the name of the operation by
     *            which you can obtain the target node from the source node.
     * 
     * @return the new created directed graph edge.
     */
    public final static IGlobalVariableEdge createGlobalVariableEdge(
            IReferenceTypeNode sourceNode, IValueNode targetNode, String name) {

        IGlobalVariableEdge result = null;

        result = new GlobalVariableEdge(sourceNode, targetNode, name);

        return result;
    }

    /**
     * Creates a directed edge from the object node to one field of this
     * reference.
     * 
     * @param sourceNode
     *            the class object reference that contains the field.
     * @param targetNode
     *            the value node that represents the field.
     * @param fieldName
     *            the name of the field
     * @param declaringTypeSignature
     *            the JNI-style signature of the declaring type of the field.
     * @param superclassField
     *            true if the field is declared in one superclass of the call
     *            object reference, otherwise false.
     * @return a directed edge from the class object reference to one field of
     *         this reference.
     */
    public final static IObjectFieldEdge createObjectFieldEdge(
            IObjectNode sourceNode, IValueNode targetNode, String fieldName,
            String declaringTypeSignature, boolean superclassField) {
        IObjectFieldEdge result = null;

        result = new ObjectFieldEdge(sourceNode, targetNode, fieldName,
                declaringTypeSignature, superclassField);

        return result;
    }

    /**
     * Create a directed edge from the root node to one local variable. A local
     * variable is a variable that is direct accesible from a stacktframe.
     * 
     * @param rootNode
     *            the root and the source node of the graph edge.
     * @param targetNode
     *            the corresponding graph node of the local variable.
     * @param stackFrame
     *            the stack frame where the local variable belongs to.
     * @param variableName
     *            the name of the local variable.
     * @return the new created directed graph edge from the root node to the
     *         graph node of the local variable.
     */
    public final static ILocalVariableEdge createLocalVarEdge(
            IRootNode rootNode, IValueNode targetNode, IStackFrame stackFrame,
            String variableName) {
        return new LocalVariableEdge(rootNode, targetNode, stackFrame,
                variableName);
    }

    /**
     * Create a directed edge from the root node to a type reference.
     * 
     * @param rootNode
     *            the root node of this graph element.
     * @param typeNode
     *            the graph node of the corresponding type reference.
     * @return the new created directed graph edge from the root node to the
     *         graph node of the type reference.
     */
    public final static ITypeEdge createTypeEdge(IRootNode rootNode,
            IReferenceTypeNode typeNode) {
        return new TypeEdge(rootNode, typeNode);
    }

    /**
     * Create a directed edge from an array reference to one of the array
     * elements.
     * 
     * @param sourceNode
     *            the graph node of the corresponding array reference.
     * @param targetNode
     *            the graph node of the corresponding array element.
     * @param arrayIndex
     *            the index of the array element within the array reference.
     * @return the new created directed graph edge from the graph node of the
     *         array reference to the graph node of the array element.
     */
    public final static IArrayElementEdge createArrayElementEdge(
            IArrayNode sourceNode, IValueNode targetNode, int arrayIndex) {
        return new ArrayElementEdge(sourceNode, targetNode, arrayIndex);
    }

    /**
     * Static factory to create a new graph node.
     * 
     * @param rootNode
     *            the root node of the new graph node.
     * @param type
     *            the type which the new graph node represents.
     * @return the generated graph node.
     * @throws MemoryGraphExtractionException
     * @throws IllegalArgumentException
     *             if the given root node or the type is null.
     * @throws IllegalArguemntException
     *             if the given type is unknown.
     */
    public final static ITypeNode createTypeNode(IRootNode rootNode,
            IJavaType type) throws MemoryGraphExtractionException {

        if (rootNode == null)
            throw new IllegalArgumentException("Rootnode must not be null.");

        if (type == null)
            throw new IllegalArgumentException("TypeNode must not be null.");

        ITypeNode result = null;

        if (type instanceof IJavaReferenceType) {
            result = new ReferenceTypeNode(rootNode, (IJavaReferenceType) type);

        } else {
            result = new PrimitiveTypeNode(rootNode, type);
        }

        return result;
    }

    /**
     * Static factory to create a new graph node.
     * 
     *  @see GraphElementUtil#createValueNode(IRootNode, IJavaValue)
     */
    public final static IValueNode createValueNode(IRootNode rootNode,
            IJavaVariable variable) throws MemoryGraphExtractionException {
        if (variable == null)
            throw new NullPointerException("Variable must not be null.");

        IJavaValue value = null;

        try {
            value = (IJavaValue) variable.getValue();
        } catch (DebugException e) {
            throw new MemoryGraphExtractionException(
                    "Cannot retrieve the value of the given variable.", e);
        }

        return GraphElementUtil.createValueNode(rootNode, value);
    }

    /**
     * Static factory to create a new graph node.
     * 
     * @param rootNode
     *            the root node of the new graph node.
     * @param value
     *            the java value of the new graph node.
     * @return the generated graph node.
     * @throws MemoryGraphExtractionException
     * @throws NullPointerException
     *             if the given root node or the value or the frame is null.
     * @throws IllegalArguemntException
     *             if the given value is unknown.
     */
    public final static IValueNode createValueNode(IRootNode rootNode,
            IJavaValue value) throws MemoryGraphExtractionException {

        if (rootNode == null)
            throw new NullPointerException("Rootnode must not be null.");

        if (value == null)
            throw new NullPointerException("Value must not be null.");

        ValueNode result = null;

        try {
            if (value instanceof IJavaPrimitiveValue) {
                result = new PrimitiveValueNode(rootNode,
                        (IJavaPrimitiveValue) value);

            } else if (value instanceof IJavaObject) {

                if (value instanceof IJavaArray) {

                    char cType = value.getJavaType().getSignature().charAt(1);

                    if (cType == Signature.C_BOOLEAN
                            || cType == Signature.C_BYTE
                            || cType == Signature.C_CHAR
                            || cType == Signature.C_DOUBLE
                            || cType == Signature.C_FLOAT
                            || cType == Signature.C_INT
                            || cType == Signature.C_LONG
                            || cType == Signature.C_SHORT) {
                        result = new GraphNodePrimitiveArrayReference(rootNode,
                                (IJavaArray) value);
                    } else {
                        result = new ArrayNode(rootNode, (IJavaArray) value);
                    }

                } else if (value instanceof IJavaObject) {
                    String referenceTypeName = null;

                    try {
                        referenceTypeName = value.getReferenceTypeName();
                    } catch (DebugException e) {
                        throw new MemoryGraphExtractionException(
                                "Cannot retrieve reference type name of the value",
                                e);
                    }

                    if (referenceTypeName.equals(String.class.getName())) {
                        result = new StringObjectNode(rootNode,
                                (IJavaObject) value);
                    } else if (referenceTypeName.equals(Class.class.getName())) {
                        result = new ClassObjectNode(rootNode,
                                (IJavaClassObject) value);

                    } else {
                        result = new ObjectNode(rootNode, (IJavaObject) value);
                    }
                }

            } else if (value.getSignature() == null) {
                result = new NullValueNode(rootNode, value);

            } else {
                throw new IllegalArgumentException("Given value ("
                        + value.getClass().getName() + ") is unknown.");
            }
        } catch (DebugException e) {
            throw new MemoryGraphExtractionException(
                    "Cannot determine the kind of the value", e);
        }

        return result;
    }
}