/*
 * $Id: ArrayNode.java,v 1.7 2006/01/03 21:02:54 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 30.08.2004 14:14:03 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.value;

import java.util.Collection;
import java.util.Iterator;

import org.deltadebugging.ddcore.DeltaSet;
import org.deltadebugging.ddstate.DDStatePlugin;
import org.deltadebugging.ddstate.comparator.delta.GraphDecorator;
import org.deltadebugging.ddstate.comparator.delta.GraphEdgeDiff;
import org.deltadebugging.ddstate.comparator.delta.GraphEdgeDiffUtil;
import org.deltadebugging.ddstate.comparator.delta.GraphEdgePair;
import org.deltadebugging.ddstate.logger.DDLogger;
import org.deltadebugging.ddstate.memorygraph.GraphElementUtil;
import org.deltadebugging.ddstate.memorygraph.IGraphEdge;
import org.deltadebugging.ddstate.memorygraph.IGraphElement;
import org.deltadebugging.ddstate.memorygraph.IGraphNode;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.deltadebugging.ddstate.memorygraph.delta.DeltaTreeFactory;
import org.deltadebugging.ddstate.memorygraph.delta.create.CreatePrimitiveValueDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdateArraySizeDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdateObjectReferenceDelta;
import org.deltadebugging.ddstate.memorygraph.exceptions.DeltaComputationException;
import org.deltadebugging.ddstate.memorygraph.exceptions.MemoryGraphExtractionException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.model.IVariable;
import org.eclipse.jdt.debug.core.IJavaArray;
import org.eclipse.jdt.debug.core.IJavaValue;
import org.eclipse.jdt.debug.core.IJavaVariable;

/**
 * COMMENTME Comment type ArrayNode
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class ArrayNode extends ObjectNode implements IArrayNode {

    /** Personal logger for this class */
    private final static DDLogger logger = DDStatePlugin
            .getLogger(ArrayNode.class);

    private int fArrayLength = -1;

    // COMMENTME
    public ArrayNode(IRootNode rootNode, IJavaArray value)
            throws MemoryGraphExtractionException {
        super(rootNode, value);

        try {
            this.fArrayLength = ((IJavaArray) this.getValue()).getLength();
        } catch (DebugException e) {
            throw new MemoryGraphExtractionException(
                    "Cannot retrieve the array length", e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.GraphNode#createSuccessors()
     */
    protected void createChildren() throws MemoryGraphExtractionException {

        IJavaValue javaValue = this.getValue();

        if (javaValue == null)
            return;

        IJavaArray value = (IJavaArray) javaValue;

        // we ask for proxys to the fields of the array, walk through them and
        // create new nodes for every field we found
        IVariable[] variables = null;
        try {
            variables = value.getVariables();
        } catch (DebugException e) {
            throw new MemoryGraphExtractionException(
                    "Problems on creating the successors of " + this.toString(),
                    e);
        }

        for (int i = 0; i < variables.length; i++) {
            if (variables[i] instanceof IJavaVariable) {
                IJavaVariable element = (IJavaVariable) variables[i];

                if (element != null) {
                    this.addArrayElement(element, i);
                }
            } else {
                logger.log(IStatus.WARNING, "Variable '"
                        + variables[i].toString()
                        + "' is not instance of IJavaVariable.");
            }
        }
    }

    public void addArrayElement(IJavaVariable variable, int arrayIndex)
            throws MemoryGraphExtractionException {

        logger.trace(DDStatePlugin.DEBUG_GRAPH_EXTRACTOR, "Add " + arrayIndex
                + "th element of " + this.getReferenceTypeName() + " array.");

        IGraphNode successor = this.createAndRegisterNode(variable);

        assert successor instanceof IValueNode;

        IGraphEdge newSuccessor = GraphElementUtil.createArrayElementEdge(this,
                (IValueNode) successor, arrayIndex);

        this.addSuccessor(newSuccessor);
        successor.addParent(newSuccessor);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.value.IArrayNode#getLength()
     */
    public int getLength() {
        return this.fArrayLength;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.IGraphElement#computeDeltaSet(org.deltadebugging.ddstate.memorygraph.IGraphElement)
     */
    public DeltaSet computeDeltaSet(IGraphElement element)
            throws DeltaComputationException {

        if (!this.isMaskedAs(GraphDecorator.NOT_COMPARED)) {
            return new DeltaSet();
        }

        DeltaSet result = new DeltaSet();

        assert element instanceof IArrayNode;
        IArrayNode arrRef = (IArrayNode) element;

        assert this.isAssociatedWith(arrRef);
        assert this.getReferenceTypeName()
                .equals(arrRef.getReferenceTypeName());

        GraphEdgeDiff diff = GraphEdgeDiffUtil.computeDiff(this, arrRef);

        // if the length of the two arrays are equal, we mask the nodes as equal
        if (this.getChildren().size() == arrRef.getChildren().size()) {
            GraphDecorator.decorateGraphElementsAsEqual(this, arrRef);

            // , else we mark the nodes as different and generate an update
            // delta to correct the length
        } else {
            GraphDecorator.decorateGraphElementsAsDifferent(this, arrRef);
            UpdateArraySizeDelta updateArraySizeDelta = new UpdateArraySizeDelta(
                    this, arrRef);

            result.add(updateArraySizeDelta);

            result.addAll(this.handleAddedElements(updateArraySizeDelta, diff
                    .getEdgesOnlyInSecondGraph()));
            result.addAll(this.handleDeletedElements(diff
                    .getEdgesOnlyInFirstGraph()));
        }

        // compute delta set of edges that are in both arrays
        Collection edgePairs = diff.getEdgesInBothGraph();
        for (Iterator itEdgePairs = edgePairs.iterator(); itEdgePairs.hasNext();) {
            GraphEdgePair edgePair = (GraphEdgePair) itEdgePairs.next();

            result.addAll(edgePair.getFirstEdge().computeDeltaSet(
                    edgePair.getSecondEdge()));
        }

        return result;
    }

    private DeltaSet handleDeletedElements(Collection edges) {
        DeltaSet result = new DeltaSet();

        for (Iterator itEdges = edges.iterator(); itEdges.hasNext();) {
            IGraphEdge edge = (IGraphEdge) itEdges.next();
            // IGraphNode targetNode = edge.getTargetNode();
            //	
            // // we cannot generate a delete delta for a null value
            // if (targetNode instanceof IObjectNode) {
            // result.add(new DeleteObjectReferenceDelta(edge));
            //	
            // } else if (targetNode instanceof IPrimitiveValueNode) {
            // result.add(new DeletePrimitiveValueDelta(
            // (IPrimitiveValueNode) targetNode));
            // }

            GraphDecorator.decorateGraphEdgeAs(edge,
                    GraphDecorator.ONLY_IN_FIRSTGRAPH, true);
        }

        return result;
    }

    private DeltaSet handleAddedElements(
            UpdateArraySizeDelta updateArraySizeDelta, Collection edges) {
        DeltaSet result = new DeltaSet();

        for (Iterator itEdges = edges.iterator(); itEdges.hasNext();) {
            IGraphEdge edge = (IGraphEdge) itEdges.next();
            IGraphNode targetNode = edge.getTargetNode();

            // we cannot generate a create delta for a null value
            if (targetNode instanceof IPrimitiveValueNode) {
                result.add(new CreatePrimitiveValueDelta(updateArraySizeDelta,
                        edge));

            } else if (!(targetNode instanceof INullValueNode)) {

                if (!targetNode.hasAssociatedElement()) {
                    result.addAll(DeltaTreeFactory.createDeltaTree(edge,
                            (IObjectNode) targetNode));

                } else {
                    result.add(new UpdateObjectReferenceDelta(edge,
                            (IObjectNode) targetNode.getAssociatedElement()));
                }
            }

            GraphDecorator.decorateGraphEdgeAs(edge,
                    GraphDecorator.ONLY_IN_SECONDGRAPH, true);
        }

        return result;
    }
}