/*
 * $Id: ObjectNode.java,v 1.7 2006/01/03 21:02:54 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 26.08.2004 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.value;

import java.util.Iterator;

import org.deltadebugging.ddcore.DeltaSet;
import org.deltadebugging.ddstate.DDStatePlugin;
import org.deltadebugging.ddstate.comparator.delta.GraphDecorator;
import org.deltadebugging.ddstate.logger.DDLogger;
import org.deltadebugging.ddstate.memorygraph.GraphElementUtil;
import org.deltadebugging.ddstate.memorygraph.IGraphEdge;
import org.deltadebugging.ddstate.memorygraph.IGraphElement;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.deltadebugging.ddstate.memorygraph.exceptions.DeltaComputationException;
import org.deltadebugging.ddstate.memorygraph.exceptions.MemoryGraphExtractionException;
import org.eclipse.debug.core.DebugException;
import org.eclipse.jdt.debug.core.IJavaClassType;
import org.eclipse.jdt.debug.core.IJavaFieldVariable;
import org.eclipse.jdt.debug.core.IJavaObject;
import org.eclipse.jdt.debug.core.IJavaValue;

/**
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 * 
 * COMMENTME Comment type ***
 */
public class ObjectNode extends ValueNode implements IObjectNode {

    /** Personal logger for this class */
    private final static DDLogger logger = DDStatePlugin
            .getLogger(ObjectNode.class);

    /** Exclude the graph nodes of this packages from unfolding */
    private final static String[] EXCLUDE_PACKAGES = new String[] {
            "java.lang.reflect", "org.apache.bcel" };

    // COMMENTME
    public ObjectNode(IRootNode rootNode, IJavaObject value)
            throws MemoryGraphExtractionException {
        super(rootNode, value);
    }

    public boolean graphEquals(IGraphElement element) {
        if (!(element instanceof IObjectNode)) {
            return false;
        }

        return super.graphEquals(element);
    }

    /**
     * Exclude the graph node, which reference type name match one of the
     * packages of the <code>EXCLUDE_PACKAGE list</code> from unfolding.
     * 
     * @return true if the reference type name of this graph node match none of
     *         the packages in the <code>EXCLUDE_PACKAGE list</code> list,
     *         otherwise false.
     */
    public boolean shouldUnfoldSubgraph() {
        boolean result = true;

        for (int i = 0; i < EXCLUDE_PACKAGES.length; i++) {
            String packageName = EXCLUDE_PACKAGES[i];
            if (this.getReferenceTypeName().startsWith(packageName)) {
                result = false;
                break;
            }
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.IGraphElement#computeDeltaSet(org.deltadebugging.ddstate.memorygraph.IGraphElement)
     */
    public DeltaSet computeDeltaSet(IGraphElement element)
            throws DeltaComputationException {

        if (!this.isMaskedAs(GraphDecorator.NOT_COMPARED)) {
            return new DeltaSet();
        }

        DeltaSet result = new DeltaSet();

        assert element instanceof IObjectNode;
        IObjectNode objRef = (IObjectNode) element;

        assert this.isAssociatedWith(objRef);
        assert this.getReferenceTypeName()
                .equals(objRef.getReferenceTypeName());

        // Both class object references has the same type, also they have the
        // same number of successors with the same names. Because the successors
        // are sorted by name, we can go through the successors and compare each
        // other.
        assert this.getChildren().size() == objRef.getChildren().size();

        // two graph node class object references are equal, if and only if they
        // have the same reference type name and are associated to each other.
        // Both criteria are preconditions of this method.
        GraphDecorator.decorateGraphElementsAsEqual(this, element);

        Iterator itFirstSuccessors = this.getChildren().iterator();
        Iterator itSecondSuccessors = objRef.getChildren().iterator();

        IGraphEdge firstSuccessor = null;
        IGraphEdge secondSuccessor = null;

        while (itFirstSuccessors.hasNext()) {
            firstSuccessor = (IGraphEdge) itFirstSuccessors.next();
            secondSuccessor = (IGraphEdge) itSecondSuccessors.next();

            result.addAll(firstSuccessor.computeDeltaSet(secondSuccessor));
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.GraphNode#createSuccessors()
     */
    protected void createChildren() throws MemoryGraphExtractionException {

        IJavaValue javaValue = this.getValue();

        if (javaValue == null)
            return;

        IJavaObject value = (IJavaObject) javaValue;

        try {
            IJavaClassType classType = ((IJavaClassType) this.getValue()
                    .getJavaType());
            boolean superClass = false;

            // analye all types (including the supertypes) for variables
            while (classType != null) {

                String[] fieldNames = classType.getDeclaredFieldNames();
                String typeSig = classType.getSignature();
                String typeName = classType.getName();

                for (int i = 0; i < fieldNames.length; i++) {
                    String fieldName = fieldNames[i];

                    IJavaFieldVariable fieldVariable = value.getField(
                            fieldName, typeSig);

                    logger.trace(DDStatePlugin.DEBUG_GRAPH_EXTRACTOR,
                            "Create a new successor of class object reference '"
                                    + this.getReferenceTypeName() + "'\n"
                                    + "\t- fieldName = '" + fieldName
                                    + "' declared in '" + typeName + "'");

                    IValueNode successor = (IValueNode) this
                            .createAndRegisterNode(fieldVariable);

                    IGraphEdge newSuccessor = GraphElementUtil
                            .createObjectFieldEdge(this, successor, fieldName,
                                    typeSig, superClass);

                    this.addSuccessor(newSuccessor);
                    successor.addParent(newSuccessor);
                }

                classType = classType.getSuperclass();
                superClass = true;
            }

        } catch (DebugException e) {
            throw new MemoryGraphExtractionException(
                    "Problems on creating the successors of " + this.toString(),
                    e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return "instance of " + this.getReferenceTypeName();
    }
}