/**
 * <p>
 * Title: Bytecode instrumentation
 * </p>
 * <p>
 * Description: Tracing framework for java bytecode
 * </p>
 * <p>
 * Copyright: Copyright (c) 2004
 * </p>
 * <p>
 * Company: Universitt Saarbrcken
 * </p>
 * 
 * @author Valentin Dallmeier
 * @version 1.0
 */
package org.softevo.sibrelib.source;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.NoSuchElementException;

public abstract class ClassFileSource {

    /**
     * Stores all <code>ClassFiles</code> that were created by this source.
     * <BR>
     */
    protected Hashtable classesCreated = null;

    /**
     * Creates a new source. <BR>
     */
    public ClassFileSource() {
        this.classesCreated = new Hashtable();
    }

    /**
     * Creates a class loader for the bcel repository to be able to load the
     * classes of this source. <BR>
     * 
     * @return the loader for the repository or <code>null</code> if the
     *         loader can't be created
     */
    public abstract ClassLoader createBCELClassLoader();

    /**
     * Gets an iterator for all <code>ClassFiles</code> in this source. <BR>
     * 
     * @return the iterator
     */
    public abstract Iterator getClassIterator();

    /**
     * Saves modified version of the source. <BR>
     * Implementations of this method do not alter the state of the stream
     * (init, close, flush etc.).
     * 
     * @throws IOException
     *             if an exception occurs writing
     */
    public abstract void saveModifiedVersion() throws IOException;

    /**
     * Gets the class path that is needed in order to load all classes from the
     * source. <BR>
     */
    public abstract String getClassPath();

    /**
     * Concrete subclasses implement this method to load classes that are not
     * already present in the cache. <BR>
     * 
     * @param className
     *            the name of the class
     * @return the <code>ClassFile</code> for the class
     * @throws NoSuchElementException
     *             if no class for the name was found
     * @throws IOException
     *             if an exception occurs reading the class
     */
    protected abstract ClassFile findClassFile(String className)
            throws NoSuchElementException, IOException;

    /**
     * Gets a <code>ClassFile</code> that corresponds to the class name
     * passed. <BR>
     * 
     * @param className
     *            the fully qualified class name of the class sought
     * @return the <code>ClassFile</code> for the class
     * @throws NoSuchElementException
     *             if no class with the name passed was found
     * @throws IOException
     *             if an exception occurs reading the class
     */
    public ClassFile getClassForName(String className)
            throws NoSuchElementException, IOException {
        ClassFile result = null;

        if (this.classesCreated.containsKey(className)) {
            result = (ClassFile) this.classesCreated.get(className);
        } else {
            ClassFile classFile = findClassFile(className);
            this.classesCreated.put(classFile.getClassName(), classFile);
            result = classFile;
        }

        return result;
    }

    /**
     * <p>
     * Title: Bytecode instrumentation
     * </p>
     * <p>
     * Description: Tracing framework for java bytecode
     * </p>
     * <p>
     * Copyright: Copyright (c) 2004
     * </p>
     * <p>
     * Company: Universitt Saarbrcken
     * </p>
     * 
     * @author Valentin Dallmeier
     * @version 1.0
     * 
     * This iterator allows subclasses to easily iterate over all classes
     * provided in the source. <BR>
     * Information about the classes in the concrete source is provided by
     * passing an iterator over all class names. This class then queries the
     * cache. In case of a miss it calls the <code>findClassFile</code> method
     * to load the class.
     */
    protected class ClassFileIterator implements Iterator {

        /**
         * The class name iterator. <BR>
         */
        private Iterator iterator = null;

        /**
         * Creates a new class iterator. <BR>
         * 
         * @param iterator
         *            the iterator for the class names
         */
        public ClassFileIterator(Iterator iterator) {
            this.iterator = iterator;
        }

        /**
         * This method first tries to load the next class from the cache. If
         * this fails, the <code>findClassFile</code> method is called. If the
         * latter call raises an exception, this method throws a
         * <code>NoSuchElementException</code>.
         * 
         * @return the next <code>ClassFile</code>
         */
        public Object next() {
            Object result = null;

            String className;
            ClassFile classFile;

            className = this.iterator.next().toString();
            if (ClassFileSource.this.classesCreated.containsKey(className)) {
                result = (ClassFile) ClassFileSource.this.classesCreated
                        .get(className);
            } else {
                try {
                    classFile = findClassFile(className);
                    ClassFileSource.this.classesCreated.put(className,
                            classFile);
                    result = classFile;
                } catch (IOException exception) {
                    throw new NoSuchElementException(
                            "Unable to read zip file entry: "
                                    + exception.getMessage());
                }
            }

            return result;
        }

        /**
         * Tests wether or not this iterator has more elements. <BR>
         * 
         * @return <code>true</code> if more elements are present,
         *         <code>false</code> otherwise
         */
        public boolean hasNext() {
            return this.iterator.hasNext();
        }

        /**
         * This method is not implemented. <BR>
         */
        public void remove() {
            throw new UnsupportedOperationException(
                    "JarClassIterator does not support remove");
        }
    }

}